@@ -25,8 +25,9 @@ from keystone.common import driver_hints
 from keystone.common import manager
 from keystone import config
 from keystone import exception
+from keystone.i18n import _
+from keystone.i18n import _LI
 from keystone import notifications
-from keystone.openstack.common.gettextutils import _
 from keystone.openstack.common import log
 
 
@@ -49,7 +50,7 @@ def calc_default_domain():
 
 @dependency.provider('assignment_api')
 @dependency.optional('revoke_api')
-@dependency.requires('credential_api', 'identity_api', 'token_api')
+@dependency.requires('credential_api', 'identity_api')
 class Manager(manager.Manager):
     """Default pivot point for the Assignment backend.
 
@@ -60,6 +61,8 @@ class Manager(manager.Manager):
     api object by both managers.
     """
     _PROJECT = 'project'
+    _ROLE_REMOVED_FROM_USER = 'role_removed_from_user'
+    _INVALIDATION_USER_PROJECT_TOKENS = 'invalidate_user_project_tokens'
 
     def __init__(self):
         assignment_driver = CONF.assignment.driver
@@ -70,6 +73,12 @@ class Manager(manager.Manager):
 
         super(Manager, self).__init__(assignment_driver)
 
+    def _get_group_ids_for_user_id(self, user_id):
+        # TODO(morganfainberg): Implement a way to get only group_ids
+        # instead of the more expensive to_dict() call for each record.
+        return [x['id'] for
+                x in self.identity_api.list_groups_for_user(user_id)]
+
     @notifications.created(_PROJECT)
     def create_project(self, tenant_id, tenant):
         tenant = tenant.copy()
@@ -83,11 +92,39 @@ class Manager(manager.Manager):
                                          ret['domain_id'])
         return ret
 
+    def assert_domain_enabled(self, domain_id, domain=None):
+        """Assert the Domain is enabled.
+
+        :raise AssertionError if domain is disabled.
+        """
+        if domain is None:
+            domain = self.get_domain(domain_id)
+        if not domain.get('enabled', True):
+            raise AssertionError(_('Domain is disabled: %s') % domain_id)
+
+    def assert_project_enabled(self, project_id, project=None):
+        """Assert the project is enabled and its associated domain is enabled.
+
+        :raise AssertionError if the project or domain is disabled.
+        """
+        if project is None:
+            project = self.get_project(project_id)
+        self.assert_domain_enabled(domain_id=project['domain_id'])
+        if not project.get('enabled', True):
+            raise AssertionError(_('Project is disabled: %s') % project_id)
+
     @notifications.disabled(_PROJECT, public=False)
-    def _disable_project(self, tenant_id):
-        return self.token_api.delete_tokens_for_users(
-            self.list_user_ids_for_project(tenant_id),
-            project_id=tenant_id)
+    def _disable_project(self, project_id):
+        """Emit a notification to the callback system project is been disabled.
+
+        This method, and associated callback listeners, removes the need for
+        making direct calls to other managers to take action (e.g. revoking
+        project scoped tokens) when a project is disabled.
+
+        :param project_id: project identifier
+        :type project_id: string
+        """
+        pass
 
     @notifications.updated(_PROJECT)
     def update_project(self, tenant_id, tenant):
@@ -95,7 +132,8 @@ class Manager(manager.Manager):
         tenant = tenant.copy()
         if 'enabled' in tenant:
             tenant['enabled'] = clean.project_enabled(tenant['enabled'])
-        if not tenant.get('enabled', True):
+        if (original_tenant.get('enabled', True) and
+                not tenant.get('enabled', True)):
             self._disable_project(tenant_id)
         ret = self.driver.update_project(tenant_id, tenant)
         self.get_project.invalidate(self, tenant_id)
@@ -106,8 +144,10 @@ class Manager(manager.Manager):
     @notifications.deleted(_PROJECT)
     def delete_project(self, tenant_id):
         project = self.driver.get_project(tenant_id)
-        user_ids = self.list_user_ids_for_project(tenant_id)
-        self.token_api.delete_tokens_for_users(user_ids, project_id=tenant_id)
+        project_user_ids = self.list_user_ids_for_project(tenant_id)
+        for user_id in project_user_ids:
+            payload = {'user_id': user_id, 'project_id': tenant_id}
+            self._emit_invalidate_user_project_tokens_notification(payload)
         ret = self.driver.delete_project(tenant_id)
         self.get_project.invalidate(self, tenant_id)
         self.get_project_by_name.invalidate(self, project['name'],
@@ -129,31 +169,11 @@ class Manager(manager.Manager):
 
         """
         def _get_group_project_roles(user_id, project_ref):
-            role_list = []
-            group_refs = self.identity_api.list_groups_for_user(user_id)
-            for x in group_refs:
-                try:
-                    metadata_ref = self._get_metadata(
-                        group_id=x['id'], tenant_id=project_ref['id'])
-                    role_list += self._roles_from_role_dicts(
-                        metadata_ref.get('roles', {}), False)
-                except exception.MetadataNotFound:
-                    # no group grant, skip
-                    pass
-
-                if CONF.os_inherit.enabled:
-                    # Now get any inherited group roles for the owning domain
-                    try:
-                        metadata_ref = self._get_metadata(
-                            group_id=x['id'],
-                            domain_id=project_ref['domain_id'])
-                        role_list += self._roles_from_role_dicts(
-                            metadata_ref.get('roles', {}), True)
-                    except (exception.MetadataNotFound,
-                            exception.NotImplemented):
-                        pass
-
-            return role_list
+            group_ids = self._get_group_ids_for_user_id(user_id)
+            return self.driver.get_group_project_roles(
+                group_ids,
+                project_ref['id'],
+                project_ref['domain_id'])
 
         def _get_user_project_roles(user_id, project_ref):
             role_list = []
@@ -194,10 +214,10 @@ class Manager(manager.Manager):
 
         def _get_group_domain_roles(user_id, domain_id):
             role_list = []
-            group_refs = self.identity_api.list_groups_for_user(user_id)
-            for x in group_refs:
+            group_ids = self._get_group_ids_for_user_id(user_id)
+            for group_id in group_ids:
                 try:
-                    metadata_ref = self._get_metadata(group_id=x['id'],
+                    metadata_ref = self._get_metadata(group_id=group_id,
                                                       domain_id=domain_id)
                     role_list += self._roles_from_role_dicts(
                         metadata_ref.get('roles', {}), False)
@@ -240,17 +260,12 @@ class Manager(manager.Manager):
                 tenant_id,
                 config.CONF.member_role_id)
         except exception.RoleNotFound:
-            LOG.info(_("Creating the default role %s "
-                       "because it does not exist."),
+            LOG.info(_LI("Creating the default role %s "
+                         "because it does not exist."),
                      config.CONF.member_role_id)
             role = {'id': CONF.member_role_id,
                     'name': CONF.member_role_name}
-            try:
-                self.driver.create_role(config.CONF.member_role_id, role)
-            except exception.Conflict:
-                LOG.info(_("Creating the default role %s failed because it "
-                           "was already created"),
-                         config.CONF.member_role_id)
+            self.driver.create_role(config.CONF.member_role_id, role)
             # now that default role exists, the add should succeed
             self.driver.add_role_to_user_and_project(
                 user_id,
@@ -277,8 +292,7 @@ class Manager(manager.Manager):
                                                     project_id=tenant_id)
 
             except exception.RoleNotFound:
-                LOG.debug(_("Removing role %s failed because it does not "
-                            "exist."),
+                LOG.debug("Removing role %s failed because it does not exist.",
                           role_id)
 
     # TODO(henry-nash): We might want to consider list limiting this at some
@@ -290,9 +304,7 @@ class Manager(manager.Manager):
         # list here and pass it in. The rest of the detailed logic of listing
         # projects for a user is pushed down into the driver to enable
         # optimization with the various backend technologies (SQL, LDAP etc.).
-
-        group_ids = [x['id'] for
-                     x in self.identity_api.list_groups_for_user(user_id)]
+        group_ids = self._get_group_ids_for_user_id(user_id)
         return self.driver.list_projects_for_user(
             user_id, group_ids, hints or driver_hints.Hints())
 
@@ -308,6 +320,9 @@ class Manager(manager.Manager):
 
     @notifications.created('domain')
     def create_domain(self, domain_id, domain):
+        if (not self.identity_api.multiple_domains_supported and
+                domain_id != CONF.identity.default_domain_id):
+            raise exception.Forbidden(_('Multiple domains are not supported'))
         domain.setdefault('enabled', True)
         domain['enabled'] = clean.domain_enabled(domain['enabled'])
         ret = self.driver.create_domain(domain_id, domain)
@@ -320,9 +335,31 @@ class Manager(manager.Manager):
     def list_domains(self, hints=None):
         return self.driver.list_domains(hints or driver_hints.Hints())
 
+    # TODO(henry-nash): We might want to consider list limiting this at some
+    # point in the future.
+    def list_domains_for_user(self, user_id, hints=None):
+        # NOTE(henry-nash): In order to get a complete list of user domains,
+        # the driver will need to look at group assignments.  To avoid cross
+        # calling between the assignment and identity driver we get the group
+        # list here and pass it in. The rest of the detailed logic of listing
+        # projects for a user is pushed down into the driver to enable
+        # optimization with the various backend technologies (SQL, LDAP etc.).
+        group_ids = self._get_group_ids_for_user_id(user_id)
+        return self.driver.list_domains_for_user(
+            user_id, group_ids, hints or driver_hints.Hints())
+
     @notifications.disabled('domain', public=False)
     def _disable_domain(self, domain_id):
-        self.token_api.delete_tokens_for_domain(domain_id)
+        """Emit a notification to the callback system domain is been disabled.
+
+        This method, and associated callback listeners, removes the need for
+        making direct calls to other managers to take action (e.g. revoking
+        domain scoped tokens) when a domain is disabled.
+
+        :param domain_id: domain identifier
+        :type domain_id: string
+        """
+        pass
 
     @notifications.updated('domain')
     def update_domain(self, domain_id, domain):
@@ -332,7 +369,8 @@ class Manager(manager.Manager):
         ret = self.driver.update_domain(domain_id, domain)
         # disable owned users & projects when the API user specifically set
         #     enabled=False
-        if not domain.get('enabled', True):
+        if (original_domain.get('enabled', True) and
+                not domain.get('enabled', True)):
             self._disable_domain(domain_id)
         self.get_domain.invalidate(self, domain_id)
         self.get_domain_by_name.invalidate(self, original_domain['name'])
@@ -382,9 +420,9 @@ class Manager(manager.Manager):
         Users: Reference domains for grants
 
         """
-        user_refs = self.identity_api.list_users()
+        user_refs = self.identity_api.list_users(domain_scope=domain_id)
         proj_refs = self.list_projects()
-        group_refs = self.identity_api.list_groups()
+        group_refs = self.identity_api.list_groups(domain_scope=domain_id)
 
         # First delete the projects themselves
         for project in proj_refs:
@@ -392,9 +430,9 @@ class Manager(manager.Manager):
                 try:
                     self.delete_project(project['id'])
                 except exception.ProjectNotFound:
-                    LOG.debug(_('Project %(projectid)s not found when '
-                                'deleting domain contents for %(domainid)s, '
-                                'continuing with cleanup.'),
+                    LOG.debug(('Project %(projectid)s not found when '
+                               'deleting domain contents for %(domainid)s, '
+                               'continuing with cleanup.'),
                               {'projectid': project['id'],
                                'domainid': domain_id})
 
@@ -402,24 +440,22 @@ class Manager(manager.Manager):
             # Cleanup any existing groups.
             if group['domain_id'] == domain_id:
                 try:
-                    self.identity_api.delete_group(group['id'],
-                                                   domain_scope=domain_id)
+                    self.identity_api.delete_group(group['id'])
                 except exception.GroupNotFound:
-                    LOG.debug(_('Group %(groupid)s not found when deleting '
-                                'domain contents for %(domainid)s, continuing '
-                                'with cleanup.'),
+                    LOG.debug(('Group %(groupid)s not found when deleting '
+                               'domain contents for %(domainid)s, continuing '
+                               'with cleanup.'),
                               {'groupid': group['id'], 'domainid': domain_id})
 
         # And finally, delete the users themselves
         for user in user_refs:
             if user['domain_id'] == domain_id:
                 try:
-                    self.identity_api.delete_user(user['id'],
-                                                  domain_scope=domain_id)
+                    self.identity_api.delete_user(user['id'])
                 except exception.UserNotFound:
-                    LOG.debug(_('User %(userid)s not found when '
-                                'deleting domain contents for %(domainid)s, '
-                                'continuing with cleanup.'),
+                    LOG.debug(('User %(userid)s not found when '
+                               'deleting domain contents for %(domainid)s, '
+                               'continuing with cleanup.'),
                               {'userid': user['id'],
                                'domainid': domain_id})
 
@@ -500,16 +536,26 @@ class Manager(manager.Manager):
     def remove_role_from_user_and_project(self, user_id, tenant_id, role_id):
         self.driver.remove_role_from_user_and_project(user_id, tenant_id,
                                                       role_id)
-        if CONF.token.revoke_by_id:
-            self.token_api.delete_tokens_for_user(user_id)
+        self.identity_api.emit_invalidate_user_token_persistence(user_id)
         if self.revoke_api:
             self.revoke_api.revoke_by_grant(role_id, user_id=user_id,
                                             project_id=tenant_id)
 
+    @notifications.internal(notifications.INVALIDATE_USER_TOKEN_PERSISTENCE)
+    def _emit_invalidate_user_token_persistence(self, user_id):
+        self.identity_api.emit_invalidate_user_token_persistence(user_id)
+
+    @notifications.role_assignment('created')
+    def create_grant(self, role_id, user_id=None, group_id=None,
+                     domain_id=None, project_id=None,
+                     inherited_to_projects=False, context=None):
+        self.driver.create_grant(role_id, user_id, group_id, domain_id,
+                                 project_id, inherited_to_projects)
+
+    @notifications.role_assignment('deleted')
     def delete_grant(self, role_id, user_id=None, group_id=None,
                      domain_id=None, project_id=None,
-                     inherited_to_projects=False):
-        user_ids = []
+                     inherited_to_projects=False, context=None):
         if group_id is None:
             if self.revoke_api:
                 self.revoke_api.revoke_by_grant(user_id=user_id,
@@ -520,23 +566,22 @@ class Manager(manager.Manager):
             try:
                 # NOTE(morganfainberg): The user ids are the important part
                 # for invalidating tokens below, so extract them here.
-                for user in self.identity_api.list_users_in_group(group_id,
-                                                                  domain_id):
+                for user in self.identity_api.list_users_in_group(group_id):
                     if user['id'] != user_id:
-                        user_ids.append(user['id'])
+                        self._emit_invalidate_user_token_persistence(
+                            user['id'])
                         if self.revoke_api:
                             self.revoke_api.revoke_by_grant(
                                 user_id=user['id'], role_id=role_id,
                                 domain_id=domain_id, project_id=project_id)
             except exception.GroupNotFound:
-                LOG.debug(_('Group %s not found, no tokens to invalidate.'),
+                LOG.debug('Group %s not found, no tokens to invalidate.',
                           group_id)
 
         self.driver.delete_grant(role_id, user_id, group_id, domain_id,
                                  project_id, inherited_to_projects)
         if user_id is not None:
-            user_ids.append(user_id)
-        self.token_api.delete_tokens_for_users(user_ids)
+            self._emit_invalidate_user_token_persistence(user_id)
 
     def _delete_tokens_for_role(self, role_id):
         assignments = self.list_role_assignments_for_role(role_id=role_id)
@@ -557,7 +602,8 @@ class Manager(manager.Manager):
                     user_and_project_ids.append(
                         (assignment['user_id'], assignment['project_id']))
                 elif 'domain_id' in assignment:
-                    user_ids.add(assignment['user_id'])
+                    self._emit_invalidate_user_token_persistence(
+                        assignment['user_id'])
             elif 'group_id' in assignment:
                 # Add in any users for this group, being tolerant of any
                 # cross-driver database integrity errors.
@@ -572,8 +618,8 @@ class Manager(manager.Manager):
                         target = _('Domain (%s)') % assignment['domain_id']
                     else:
                         target = _('Unknown Target')
-                    msg = _('Group (%(group)s), referenced in assignment '
-                            'for %(target)s, not found - ignoring.')
+                    msg = ('Group (%(group)s), referenced in assignment '
+                           'for %(target)s, not found - ignoring.')
                     LOG.debug(msg, {'group': assignment['group_id'],
                                     'target': target})
                     continue
@@ -584,7 +630,8 @@ class Manager(manager.Manager):
                             (user['id'], assignment['project_id']))
                 elif 'domain_id' in assignment:
                     for user in users:
-                        user_ids.add(user['id'])
+                        self._emit_invalidate_user_token_persistence(
+                            user['id'])
 
         # Now process the built up lists.  Before issuing calls to delete any
         # tokens, let's try and minimize the number of calls by pruning out
@@ -595,9 +642,18 @@ class Manager(manager.Manager):
             if user_and_project_id[0] not in user_ids:
                 user_and_project_ids_to_action.append(user_and_project_id)
 
-        self.token_api.delete_tokens_for_users(user_ids)
         for user_id, project_id in user_and_project_ids_to_action:
-            self.token_api.delete_tokens_for_user(user_id, project_id)
+            self._emit_invalidate_user_project_tokens_notification(
+                {'user_id': user_id,
+                 'project_id': project_id})
+
+    @notifications.internal(
+        notifications.INVALIDATE_USER_PROJECT_TOKEN_PERSISTENCE)
+    def _emit_invalidate_user_project_tokens_notification(self, payload):
+        # This notification's payload is a dict of user_id and
+        # project_id so the token provider can invalidate the tokens
+        # from persistence if persistence is enabled.
+        pass
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -648,7 +704,7 @@ class Driver(object):
         :raises: keystone.exception.ProjectNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def list_user_ids_for_project(self, tenant_id):
@@ -658,7 +714,7 @@ class Driver(object):
         :raises: keystone.exception.ProjectNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def add_role_to_user_and_project(self, user_id, tenant_id, role_id):
@@ -668,7 +724,7 @@ class Driver(object):
                  keystone.exception.ProjectNotFound,
                  keystone.exception.RoleNotFound
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def remove_role_from_user_and_project(self, user_id, tenant_id, role_id):
@@ -679,7 +735,7 @@ class Driver(object):
                  keystone.exception.RoleNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     # assignment/grant crud
 
@@ -698,7 +754,7 @@ class Driver(object):
                  keystone.exception.RoleNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def list_grants(self, user_id=None, group_id=None,
@@ -713,7 +769,7 @@ class Driver(object):
                  keystone.exception.RoleNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def get_grant(self, role_id, user_id=None, group_id=None,
@@ -728,7 +784,7 @@ class Driver(object):
                  keystone.exception.RoleNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def delete_grant(self, role_id, user_id=None, group_id=None,
@@ -741,12 +797,12 @@ class Driver(object):
                  keystone.exception.RoleNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def list_role_assignments(self):
 
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     # domain crud
     @abc.abstractmethod
@@ -756,7 +812,7 @@ class Driver(object):
         :raises: keystone.exception.Conflict
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def list_domains(self, hints):
@@ -768,7 +824,7 @@ class Driver(object):
         :returns: a list of domain_refs or an empty list.
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def get_domain(self, domain_id):
@@ -778,7 +834,7 @@ class Driver(object):
         :raises: keystone.exception.DomainNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def get_domain_by_name(self, domain_name):
@@ -788,7 +844,7 @@ class Driver(object):
         :raises: keystone.exception.DomainNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def update_domain(self, domain_id, domain):
@@ -798,7 +854,7 @@ class Driver(object):
                  keystone.exception.Conflict
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def delete_domain(self, domain_id):
@@ -807,7 +863,7 @@ class Driver(object):
         :raises: keystone.exception.DomainNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     # project crud
     @abc.abstractmethod
@@ -817,7 +873,7 @@ class Driver(object):
         :raises: keystone.exception.Conflict
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def list_projects(self, hints):
@@ -829,7 +885,7 @@ class Driver(object):
         :returns: a list of project_refs or an empty list.
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def list_projects_in_domain(self, domain_id):
@@ -841,7 +897,7 @@ class Driver(object):
         :returns: a list of project_refs or an empty list.
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def list_projects_for_user(self, user_id, group_ids, hints):
@@ -857,7 +913,7 @@ class Driver(object):
         :returns: a list of project_refs or an empty list.
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def get_roles_for_groups(self, group_ids, project_id=None, domain_id=None):
@@ -878,7 +934,7 @@ class Driver(object):
                   project_id or domain_id
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def list_projects_for_groups(self, group_ids):
@@ -888,7 +944,23 @@ class Driver(object):
         :returns: List of projects accessible to specified groups.
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
+
+    @abc.abstractmethod
+    def list_domains_for_user(self, user_id, group_ids, hints):
+        """List all domains associated with a given user.
+
+        :param user_id: the user in question
+        :param group_ids: the groups this user is a member of.  This list is
+                          built in the Manager, so that the driver itself
+                          does not have to call across to identity.
+        :param hints: filter hints which the driver should
+                      implement if at all possible.
+
+        :returns: a list of domain_refs or an empty list.
+
+        """
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def list_domains_for_groups(self, group_ids):
@@ -898,7 +970,7 @@ class Driver(object):
         :returns: List of domains accessible to specified groups.
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def get_project(self, project_id):
@@ -908,7 +980,7 @@ class Driver(object):
         :raises: keystone.exception.ProjectNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def update_project(self, project_id, project):
@@ -918,7 +990,7 @@ class Driver(object):
                  keystone.exception.Conflict
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def delete_project(self, project_id):
@@ -927,7 +999,7 @@ class Driver(object):
         :raises: keystone.exception.ProjectNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     # role crud
 
@@ -938,7 +1010,7 @@ class Driver(object):
         :raises: keystone.exception.Conflict
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def list_roles(self, hints):
@@ -949,6 +1021,24 @@ class Driver(object):
 
         :returns: a list of role_refs or an empty list.
 
+        """
+        raise exception.NotImplemented()  # pragma: no cover
+
+    @abc.abstractmethod
+    def get_group_project_roles(self, groups, project_id, project_domain_id):
+        """Get group roles for a specific project.
+
+        Supports the ``OS-INHERIT`` role inheritance from the project's domain
+        if supported by the assignment driver.
+
+        :param groups: list of group ids
+        :type groups: list
+        :param project_id: project identifier
+        :type project_id: str
+        :param project_domain_id: project's domain identifier
+        :type project_domain_id: str
+        :returns: list of role_refs for the project
+        :rtype: list
         """
         raise exception.NotImplemented()
 
@@ -960,7 +1050,7 @@ class Driver(object):
         :raises: keystone.exception.RoleNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def update_role(self, role_id, role):
@@ -970,7 +1060,7 @@ class Driver(object):
                  keystone.exception.Conflict
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def delete_role(self, role_id):
@@ -979,9 +1069,9 @@ class Driver(object):
         :raises: keystone.exception.RoleNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
-#TODO(ayoung): determine what else these two functions raise
+# TODO(ayoung): determine what else these two functions raise
     @abc.abstractmethod
     def delete_user(self, user_id):
         """Deletes all assignments for a user.
@@ -989,7 +1079,7 @@ class Driver(object):
         :raises: keystone.exception.RoleNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def delete_group(self, group_id):
@@ -998,11 +1088,11 @@ class Driver(object):
         :raises: keystone.exception.RoleNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
-    #domain management functions for backends that only allow a single domain.
-    #currently, this is only LDAP, but might be used by PAM or other backends
-    #as well.  This is used by both identity and assignment drivers.
+    # Domain management functions for backends that only allow a single
+    # domain.  Currently, this is only LDAP, but might be used by other
+    # backends in the future.
     def _set_default_domain(self, ref):
         """If the domain ID has not been set, set it to the default."""
         if isinstance(ref, dict):
