@@ -14,6 +14,7 @@
 
 
 from keystone import catalog
+from keystone.common import driver_hints
 from keystone.common import kvs
 
 
@@ -30,7 +31,7 @@ class Catalog(kvs.Base, catalog.Driver):
         Recursively delete any region that has the supplied region
         as its parent.
         """
-        children = [r for r in self.list_regions()
+        children = [r for r in self.list_regions(driver_hints.Hints())
                     if r['parent_region_id'] == region_id]
         for child in children:
             self._delete_child_regions(child['id'])
@@ -51,6 +52,7 @@ class Catalog(kvs.Base, catalog.Driver):
     def create_region(self, region):
         region_id = region['id']
         region.setdefault('parent_region_id')
+        region.setdefault('url')
         self._check_parent_region(region)
         self.db.set('region-%s' % region_id, region)
         region_list = set(self.db.get('region_list', []))
@@ -58,7 +60,7 @@ class Catalog(kvs.Base, catalog.Driver):
         self.db.set('region_list', list(region_list))
         return region
 
-    def list_regions(self):
+    def list_regions(self, hints):
         return [self.get_region(x) for x in self.db.get('region_list', [])]
 
     def get_region(self, region_id):
