@@ -19,7 +19,7 @@ limited to encoding and decoding base64 and it's variants. It is often
 useful to be able to perform other operations on base64 text. This
 module is meant to be used in conjunction with the core base64 module.
 
-Standarized base64 is defined in
+Standardized base64 is defined in
 RFC-4648 "The Base16, Base32, and Base64 Data Encodings".
 
 This module provides the following base64 utility functionality:
@@ -43,7 +43,7 @@ import string
 import six
 from six.moves import urllib
 
-from keystone.openstack.common.gettextutils import _
+from keystone.i18n import _
 
 
 class InvalidBase64Error(ValueError):
@@ -120,7 +120,7 @@ def filter_formatting(text):
 def base64_to_base64url(text):
     """Convert base64 text to base64url text.
 
-    base64url text is designed to be safe for use in filenames and
+    base64url text is designed to be safe for use in file names and
     URL's. It is defined in RFC-4648 Section 5.
 
     base64url differs from base64 in the last two alphabet characters
@@ -140,7 +140,7 @@ def base64_to_base64url(text):
         percent-encode
             percent-encode the pad character (e.g. '=' becomes
             '%3D'). This makes the base64url text fully safe. But
-            percent-enconding has the downside of requiring
+            percent-encoding has the downside of requiring
             percent-decoding prior to feeding the base64url text into a
             base64url decoder since most base64url decoders do not
             recognize %3D as a pad character and most decoders require
@@ -154,7 +154,7 @@ def base64_to_base64url(text):
             trailing pad character(s) for use in URL's it can be added back
             using the base64_assure_padding() function.
 
-        This function makes no decisions about which padding methodolgy to
+        This function makes no decisions about which padding methodology to
         use. One can either call base64_strip_padding() to remove any pad
         characters (restoring later with base64_assure_padding()) or call
         base64url_percent_encode() to percent-encode the pad characters.
@@ -293,14 +293,14 @@ def base64_strip_padding(text, pad='='):
 def base64_assure_padding(text, pad='='):
     """Assure the input text ends with padding.
 
-    Base64 text is normally expected to be a multple of 4
+    Base64 text is normally expected to be a multiple of 4
     characters. Each 4 character base64 sequence produces 3 octets of
     binary data. If the binary data is not a multiple of 3 the base64
-    text is padded at the end with a pad character such that is is
-    always a multple of 4. Padding is ignored and does not alter the
+    text is padded at the end with a pad character such that it is
+    always a multiple of 4. Padding is ignored and does not alter the
     binary data nor it's length.
 
-    In some circumstances is is desirable to omit the padding
+    In some circumstances it is desirable to omit the padding
     character due to transport encoding conflicts. Base64 text can
     still be correctly decoded if the length of the base64 text
     (consisting only of characters in the desired base64 alphabet) is
@@ -310,7 +310,7 @@ def base64_assure_padding(text, pad='='):
     format RFC compliant base64, this function performs this action.
 
     Input is assumed to consist only of members of a base64
-    alphabet (i.e no whitepace). Iteration yields a sequence of lines.
+    alphabet (i.e no whitespace). Iteration yields a sequence of lines.
     The line does NOT terminate with a line ending.
 
     Use the filter_formatting() function to assure the input text
@@ -347,7 +347,7 @@ def base64_wrap_iter(text, width=64):
     """Fold text into lines of text with max line length.
 
     Input is assumed to consist only of members of a base64
-    alphabet (i.e no whitepace). Iteration yields a sequence of lines.
+    alphabet (i.e no whitespace). Iteration yields a sequence of lines.
     The line does NOT terminate with a line ending.
 
     Use the filter_formatting() function to assure the input text
@@ -369,7 +369,7 @@ def base64_wrap(text, width=64):
     """Fold text into lines of text with max line length.
 
     Input is assumed to consist only of members of a base64
-    alphabet (i.e no whitepace). Fold the text into lines whose
+    alphabet (i.e no whitespace). Fold the text into lines whose
     line length is width chars long, terminate each line with line
     ending (default is '\\n'). Return the wrapped text as a single
     string.
