@@ -26,7 +26,9 @@ import eventlet
 import eventlet.wsgi
 import greenlet
 
-from keystone.openstack.common.gettextutils import _
+from keystone.i18n import _
+from keystone.i18n import _LE
+from keystone.i18n import _LI
 from keystone.openstack.common import log
 
 
@@ -69,51 +71,75 @@ class Server(object):
         self.cert_required = False
         self.keepalive = keepalive
         self.keepidle = keepidle
+        self.socket = None
 
-    def start(self, key=None, backlog=128):
-        """Run a WSGI server with the given application."""
-        LOG.info(_('Starting %(arg0)s on %(host)s:%(port)s'),
-                 {'arg0': sys.argv[0],
-                  'host': self.host,
-                  'port': self.port})
+    def listen(self, key=None, backlog=128):
+        """Create and start listening on socket.
+
+        Call before forking worker processes.
+
+        Raises Exception if this has already been called.
+        """
 
         # TODO(dims): eventlet's green dns/socket module does not actually
         # support IPv6 in getaddrinfo(). We need to get around this in the
-        # future or monitor upstream for a fix
+        # future or monitor upstream for a fix.
+        # Please refer below link
+        # (https://bitbucket.org/eventlet/eventlet/
+        # src/e0f578180d7d82d2ed3d8a96d520103503c524ec/eventlet/support/
+        # greendns.py?at=0.12#cl-163)
         info = socket.getaddrinfo(self.host,
                                   self.port,
                                   socket.AF_UNSPEC,
                                   socket.SOCK_STREAM)[0]
-        _socket = eventlet.listen(info[-1],
-                                  family=info[0],
-                                  backlog=backlog)
+
+        try:
+            self.socket = eventlet.listen(info[-1], family=info[0],
+                                          backlog=backlog)
+        except EnvironmentError:
+            LOG.error(_LE("Could not bind to %(host)s:%(port)s"),
+                      {'host': self.host, 'port': self.port})
+            raise
+
+        LOG.info(_LI('Starting %(arg0)s on %(host)s:%(port)s'),
+                 {'arg0': sys.argv[0],
+                  'host': self.host,
+                  'port': self.port})
+
+    def start(self, key=None, backlog=128):
+        """Run a WSGI server with the given application."""
+
+        if self.socket is None:
+            self.listen(key=key, backlog=backlog)
+
+        dup_socket = self.socket.dup()
         if key:
-            self.socket_info[key] = _socket.getsockname()
+            self.socket_info[key] = self.socket.getsockname()
         # SSL is enabled
         if self.do_ssl:
             if self.cert_required:
                 cert_reqs = ssl.CERT_REQUIRED
             else:
                 cert_reqs = ssl.CERT_NONE
-            sslsocket = eventlet.wrap_ssl(_socket, certfile=self.certfile,
-                                          keyfile=self.keyfile,
-                                          server_side=True,
-                                          cert_reqs=cert_reqs,
-                                          ca_certs=self.ca_certs)
-            _socket = sslsocket
+
+            dup_socket = eventlet.wrap_ssl(dup_socket, certfile=self.certfile,
+                                           keyfile=self.keyfile,
+                                           server_side=True,
+                                           cert_reqs=cert_reqs,
+                                           ca_certs=self.ca_certs)
 
         # Optionally enable keepalive on the wsgi socket.
         if self.keepalive:
-            _socket.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
+            dup_socket.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
 
             # This option isn't available in the OS X version of eventlet
             if hasattr(socket, 'TCP_KEEPIDLE') and self.keepidle is not None:
-                _socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPIDLE,
-                                   self.keepidle)
+                dup_socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPIDLE,
+                                      self.keepidle)
 
         self.greenthread = self.pool.spawn(self._run,
                                            self.application,
-                                           _socket)
+                                           dup_socket)
 
     def set_ssl(self, certfile, keyfile=None, ca_certs=None,
                 cert_required=True):
@@ -123,7 +149,7 @@ class Server(object):
         self.cert_required = cert_required
         self.do_ssl = True
 
-    def kill(self):
+    def stop(self):
         if self.greenthread is not None:
             self.greenthread.kill()
 
@@ -136,6 +162,17 @@ class Server(object):
         except greenlet.GreenletExit:
             pass
 
+    def reset(self):
+        """Required by the service interface.
+
+        The service interface is used by the launcher when receiving a
+        SIGHUP. The service interface is defined in
+        keystone.openstack.common.service.Service.
+
+        Keystone does not need to do anything here.
+        """
+        pass
+
     def _run(self, application, socket):
         """Start a WSGI server in a new green thread."""
         logger = log.getLogger('eventlet.wsgi.server')
@@ -143,6 +180,9 @@ class Server(object):
             eventlet.wsgi.server(socket, application, custom_pool=self.pool,
                                  log=EventletFilteringLogger(logger),
                                  debug=False)
+        except greenlet.GreenletExit:
+            # Wait until all servers have completed running
+            pass
         except Exception:
             LOG.exception(_('Server error'))
             raise
