@@ -14,7 +14,6 @@
 
 import sqlalchemy as sql
 
-
 ASSIGNMENT_TABLE = 'assignment'
 
 
@@ -25,3 +24,12 @@ def upgrade(migrate_engine):
     assignment = sql.Table(ASSIGNMENT_TABLE, meta, autoload=True)
     idx = sql.Index('ix_actor_id', assignment.c.actor_id)
     idx.create(migrate_engine)
+
+
+def downgrade(migrate_engine):
+    meta = sql.MetaData()
+    meta.bind = migrate_engine
+
+    assignment = sql.Table(ASSIGNMENT_TABLE, meta, autoload=True)
+    idx = sql.Index('ix_actor_id', assignment.c.actor_id)
+    idx.drop(migrate_engine)
