@@ -28,15 +28,17 @@ name = {
     'maxLength': 255
 }
 
-hex_uuid = {
+id_string = {
     'type': 'string',
-    'maxLength': 32,
-    'minLength': 32,
-    'pattern': '^[a-fA-F0-9]*$'
+    'minLength': 1,
+    'maxLength': 64,
+    # TODO(lbragstad): Find a way to make this configurable such that the end
+    # user chooses how much control they want over id_strings with a regex
+    'pattern': '^[a-zA-Z0-9-]+$'
 }
 
 description = {
-    'type': ['string', 'null']
+    'type': 'string'
 }
 
 url = {
