@@ -12,17 +12,27 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import functools
+
+from keystone.common import json_home
 from keystone.common import wsgi
 from keystone.contrib.example import controllers
 
 
-class ExampleRouter(wsgi.ExtensionRouter):
+build_resource_relation = functools.partial(
+    json_home.build_v3_extension_resource_relation,
+    extension_name='OS-EXAMPLE', extension_version='1.0')
+
+
+class ExampleRouter(wsgi.V3ExtensionRouter):
 
     PATH_PREFIX = '/OS-EXAMPLE'
 
     def add_routes(self, mapper):
         example_controller = controllers.ExampleV3Controller()
-        mapper.connect(self.PATH_PREFIX + '/example',
-                       controller=example_controller,
-                       action='do_something',
-                       conditions=dict(method=['GET']))
+
+        self._add_resource(
+            mapper, example_controller,
+            path=self.PATH_PREFIX + '/example',
+            get_action='do_something',
+            rel=build_resource_relation(resource_name='example'))
