@@ -10,4 +10,4 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystone.contrib.revoke.core import *  # flake8: noqa
+from keystone.contrib.revoke.core import *  # noqa
