@@ -18,15 +18,14 @@ import ldap
 import ldap.filter
 
 from keystone import clean
-from keystone.common import dependency
 from keystone.common import driver_hints
 from keystone.common import ldap as common_ldap
 from keystone.common import models
 from keystone.common import utils
 from keystone import config
 from keystone import exception
+from keystone.i18n import _
 from keystone import identity
-from keystone.openstack.common.gettextutils import _
 from keystone.openstack.common import log
 
 
@@ -34,7 +33,6 @@ CONF = config.CONF
 LOG = log.getLogger(__name__)
 
 
-@dependency.requires('assignment_api')
 class Identity(identity.Driver):
     def __init__(self, conf=None):
         super(Identity, self).__init__()
@@ -49,6 +47,9 @@ class Identity(identity.Driver):
     def is_domain_aware(self):
         return False
 
+    def generates_uuids(self):
+        return False
+
     # Identity interface
 
     def authenticate(self, user_id, password):
@@ -60,8 +61,8 @@ class Identity(identity.Driver):
             raise AssertionError(_('Invalid user / password'))
         conn = None
         try:
-            conn = self.user.get_connection(self.user._id_to_dn(user_id),
-                                            password)
+            conn = self.user.get_connection(user_ref['dn'],
+                                            password, end_user_auth=True)
             if not conn:
                 raise AssertionError(_('Invalid user / password'))
         except Exception:
@@ -69,13 +70,13 @@ class Identity(identity.Driver):
         finally:
             if conn:
                 conn.unbind_s()
-        return identity.filter_user(user_ref)
+        return self.user.filter_attributes(user_ref)
 
     def _get_user(self, user_id):
         return self.user.get(user_id)
 
     def get_user(self, user_id):
-        return identity.filter_user(self._get_user(user_id))
+        return self.user.get_filtered(user_id)
 
     def list_users(self, hints):
         return self.user.get_all_filtered()
@@ -83,105 +84,106 @@ class Identity(identity.Driver):
     def get_user_by_name(self, user_name, domain_id):
         # domain_id will already have been handled in the Manager layer,
         # parameter left in so this matches the Driver specification
-        return identity.filter_user(self.user.get_by_name(user_name))
+        return self.user.filter_attributes(self.user.get_by_name(user_name))
 
     # CRUD
     def create_user(self, user_id, user):
         self.user.check_allow_create()
         user_ref = self.user.create(user)
-        return identity.filter_user(user_ref)
+        return self.user.filter_attributes(user_ref)
 
     def update_user(self, user_id, user):
         self.user.check_allow_update()
-        if 'id' in user and user['id'] != user_id:
-            raise exception.ValidationError(_('Cannot change user ID'))
         old_obj = self.user.get(user_id)
         if 'name' in user and old_obj.get('name') != user['name']:
             raise exception.Conflict(_('Cannot change user name'))
 
-        user = utils.hash_ldap_user_password(user)
         if self.user.enabled_mask:
             self.user.mask_enabled_attribute(user)
+        elif self.user.enabled_invert and not self.user.enabled_emulation:
+            # We need to invert the enabled value for the old model object
+            # to prevent the LDAP update code from thinking that the enabled
+            # values are already equal.
+            user['enabled'] = not user['enabled']
+            old_obj['enabled'] = not old_obj['enabled']
+
         self.user.update(user_id, user, old_obj)
         return self.user.get_filtered(user_id)
 
     def delete_user(self, user_id):
         self.user.check_allow_delete()
-        self.assignment_api.delete_user(user_id)
-        user_dn = self.user._id_to_dn(user_id)
+        user = self.user.get(user_id)
+        user_dn = user['dn']
         groups = self.group.list_user_groups(user_dn)
         for group in groups:
             self.group.remove_user(user_dn, group['id'], user_id)
 
-        user = self.user.get(user_id)
         if hasattr(user, 'tenant_id'):
-            self.project.remove_user(user.tenant_id,
-                                     self.user._id_to_dn(user_id))
+            self.project.remove_user(user.tenant_id, user_dn)
         self.user.delete(user_id)
 
     def create_group(self, group_id, group):
         self.group.check_allow_create()
         group['name'] = clean.group_name(group['name'])
-        return self.group.create(group)
+        return common_ldap.filter_entity(self.group.create(group))
 
     def get_group(self, group_id):
-        return self.group.get(group_id)
+        return self.group.get_filtered(group_id)
 
     def update_group(self, group_id, group):
         self.group.check_allow_update()
         if 'name' in group:
             group['name'] = clean.group_name(group['name'])
-        return self.group.update(group_id, group)
+        return common_ldap.filter_entity(self.group.update(group_id, group))
 
     def delete_group(self, group_id):
         self.group.check_allow_delete()
         return self.group.delete(group_id)
 
     def add_user_to_group(self, user_id, group_id):
-        self.get_user(user_id)
-        self.get_group(group_id)
-        user_dn = self.user._id_to_dn(user_id)
+        user_ref = self._get_user(user_id)
+        user_dn = user_ref['dn']
         self.group.add_user(user_dn, group_id, user_id)
 
     def remove_user_from_group(self, user_id, group_id):
-        self.get_user(user_id)
-        self.get_group(group_id)
-        user_dn = self.user._id_to_dn(user_id)
+        user_ref = self._get_user(user_id)
+        user_dn = user_ref['dn']
         self.group.remove_user(user_dn, group_id, user_id)
 
     def list_groups_for_user(self, user_id, hints):
-        self.get_user(user_id)
-        user_dn = self.user._id_to_dn(user_id)
-        return self.group.list_user_groups(user_dn)
+        user_ref = self._get_user(user_id)
+        user_dn = user_ref['dn']
+        return self.group.list_user_groups_filtered(user_dn)
 
     def list_groups(self, hints):
-        return self.group.get_all()
+        return self.group.get_all_filtered()
 
     def list_users_in_group(self, group_id, hints):
-        self.get_group(group_id)
         users = []
         for user_dn in self.group.list_group_users(group_id):
             user_id = self.user._dn_to_id(user_dn)
             try:
                 users.append(self.user.get_filtered(user_id))
             except exception.UserNotFound:
-                LOG.debug(_("Group member '%(user_dn)s' not found in"
-                            " '%(group_id)s'. The user should be removed"
-                            " from the group. The user will be ignored."),
+                LOG.debug(("Group member '%(user_dn)s' not found in"
+                           " '%(group_id)s'. The user should be removed"
+                           " from the group. The user will be ignored."),
                           dict(user_dn=user_dn, group_id=group_id))
         return users
 
     def check_user_in_group(self, user_id, group_id):
-        self.get_user(user_id)
-        self.get_group(group_id)
         user_refs = self.list_users_in_group(group_id, driver_hints.Hints())
-        found = False
         for x in user_refs:
             if x['id'] == user_id:
-                found = True
                 break
-        if not found:
-            raise exception.NotFound(_('User not found in group'))
+        else:
+            # Try to fetch the user to see if it even exists.  This
+            # will raise a more accurate exception.
+            self.get_user(user_id)
+            raise exception.NotFound(_("User '%(user_id)s' not found in"
+                                       " group '%(group_id)s'") %
+                                     {'user_id': user_id,
+                                      'group_id': group_id})
 
 
 # TODO(termie): turn this into a data object and move logic to driver
@@ -205,6 +207,8 @@ class UserApi(common_ldap.EnabledEmuMixIn, common_ldap.BaseLdap):
         super(UserApi, self).__init__(conf)
         self.enabled_mask = conf.ldap.user_enabled_mask
         self.enabled_default = conf.ldap.user_enabled_default
+        self.enabled_invert = conf.ldap.user_enabled_invert
+        self.enabled_emulation = conf.ldap.user_enabled_emulation
 
     def _ldap_res_to_model(self, res):
         obj = super(UserApi, self)._ldap_res_to_model(res)
@@ -212,6 +216,18 @@ class UserApi(common_ldap.EnabledEmuMixIn, common_ldap.BaseLdap):
             enabled = int(obj.get('enabled', self.enabled_default))
             obj['enabled'] = ((enabled & self.enabled_mask) !=
                               self.enabled_mask)
+        elif self.enabled_invert and not self.enabled_emulation:
+            # This could be a bool or a string.  If it's a string,
+            # we need to convert it so we can invert it properly.
+            enabled = obj.get('enabled', self.enabled_default)
+            if type(enabled) is str:
+                if enabled.lower == 'true':
+                    enabled = True
+                else:
+                    enabled = False
+            obj['enabled'] = not enabled
+        obj['dn'] = res[0]
+
         return obj
 
     def mask_enabled_attribute(self, values):
@@ -224,12 +240,18 @@ class UserApi(common_ldap.EnabledEmuMixIn, common_ldap.BaseLdap):
         del values['enabled_nomask']
 
     def create(self, values):
-        values = utils.hash_ldap_user_password(values)
         if self.enabled_mask:
             orig_enabled = values['enabled']
             self.mask_enabled_attribute(values)
+        elif self.enabled_invert and not self.enabled_emulation:
+            orig_enabled = values['enabled']
+            if orig_enabled is not None:
+                values['enabled'] = not orig_enabled
+            else:
+                values['enabled'] = self.enabled_default
         values = super(UserApi, self).create(values)
-        if self.enabled_mask:
+        if self.enabled_mask or (self.enabled_invert and
+                                 not self.enabled_emulation):
             values['enabled'] = orig_enabled
         return values
 
@@ -239,10 +261,23 @@ class UserApi(common_ldap.EnabledEmuMixIn, common_ldap.BaseLdap):
 
     def get_filtered(self, user_id):
         user = self.get(user_id)
-        return identity.filter_user(user)
+        return self.filter_attributes(user)
 
     def get_all_filtered(self):
-        return [identity.filter_user(user) for user in self.get_all()]
+        return [self.filter_attributes(user) for user in self.get_all()]
+
+    def filter_attributes(self, user):
+        return identity.filter_user(common_ldap.filter_entity(user))
+
+    def is_user(self, dn):
+        """Returns True if the entry is a user."""
+
+        # NOTE(blk-u): It's easy to check if the DN is under the User tree,
+        # but may not be accurate. A more accurate test would be to fetch the
+        # entry to see if it's got the user objectclass, but this could be
+        # really expensive considering how this is used.
+
+        return common_ldap.dn_startswith(dn, self.tree_dn)
 
 
 class GroupApi(common_ldap.BaseLdap):
@@ -258,6 +293,11 @@ class GroupApi(common_ldap.BaseLdap):
     immutable_attrs = ['name']
     model = models.Group
 
+    def _ldap_res_to_model(self, res):
+        model = super(GroupApi, self)._ldap_res_to_model(res)
+        model['dn'] = res[0]
+        return model
+
     def __init__(self, conf):
         super(GroupApi, self).__init__(conf)
         self.member_attribute = (getattr(conf.ldap, 'group_member_attribute')
@@ -278,20 +318,10 @@ class GroupApi(common_ldap.BaseLdap):
             # TODO(spzala): this is only placeholder for group and domain
             # role support which will be added under bug 1101287
 
-            query = '(objectClass=%s)' % self.object_class
-            dn = None
-            dn = self._id_to_dn(group_id)
-            if dn:
-                try:
-                    conn = self.get_connection()
-                    roles = conn.search_s(dn, ldap.SCOPE_ONELEVEL,
-                                          query, ['%s' % '1.1'])
-                    for role_dn, _ in roles:
-                        conn.delete_s(role_dn)
-                except ldap.NO_SUCH_OBJECT:
-                    pass
-                finally:
-                    conn.unbind_s()
+            group_ref = self.get(group_id)
+            group_dn = group_ref['dn']
+            if group_dn:
+                self._delete_tree_nodes(group_dn, ldap.SCOPE_ONELEVEL)
             super(GroupApi, self).delete(group_id)
 
     def update(self, group_id, values):
@@ -299,32 +329,22 @@ class GroupApi(common_ldap.BaseLdap):
         return super(GroupApi, self).update(group_id, values, old_obj)
 
     def add_user(self, user_dn, group_id, user_id):
-        conn = self.get_connection()
+        group_ref = self.get(group_id)
+        group_dn = group_ref['dn']
         try:
-            conn.modify_s(
-                self._id_to_dn(group_id),
-                [(ldap.MOD_ADD,
-                  self.member_attribute,
-                  user_dn)])
-        except ldap.TYPE_OR_VALUE_EXISTS:
+            super(GroupApi, self).add_member(user_dn, group_dn)
+        except exception.Conflict:
             raise exception.Conflict(_(
                 'User %(user_id)s is already a member of group %(group_id)s') %
                 {'user_id': user_id, 'group_id': group_id})
-        finally:
-            conn.unbind_s()
 
     def remove_user(self, user_dn, group_id, user_id):
-        conn = self.get_connection()
+        group_ref = self.get(group_id)
+        group_dn = group_ref['dn']
         try:
-            conn.modify_s(
-                self._id_to_dn(group_id),
-                [(ldap.MOD_DELETE,
-                  self.member_attribute,
-                  user_dn)])
+            super(GroupApi, self).remove_member(user_dn, group_dn)
         except ldap.NO_SUCH_ATTRIBUTE:
             raise exception.UserNotFound(user_id=user_id)
-        finally:
-            conn.unbind_s()
 
     def list_user_groups(self, user_dn):
         """Return a list of groups for which the user is a member."""
@@ -334,27 +354,42 @@ class GroupApi(common_ldap.BaseLdap):
                                                   self.member_attribute,
                                                   user_dn_esc,
                                                   self.ldap_filter or '')
-        memberships = self.get_all(query)
-        return memberships
+        return self.get_all(query)
+
+    def list_user_groups_filtered(self, user_dn):
+        """Return a filtered list of groups for which the user is a member."""
+
+        user_dn_esc = ldap.filter.escape_filter_chars(user_dn)
+        query = '(&(objectClass=%s)(%s=%s)%s)' % (self.object_class,
+                                                  self.member_attribute,
+                                                  user_dn_esc,
+                                                  self.ldap_filter or '')
+        return self.get_all_filtered(query)
 
     def list_group_users(self, group_id):
         """Return a list of user dns which are members of a group."""
-        query = '(objectClass=%s)' % self.object_class
-        conn = self.get_connection()
-        group_dn = self._id_to_dn(group_id)
+        group_ref = self.get(group_id)
+        group_dn = group_ref['dn']
+
         try:
-            attrs = conn.search_s(group_dn,
-                                  ldap.SCOPE_BASE,
-                                  query, ['%s' % self.member_attribute])
+            attrs = self._ldap_get_list(group_dn, ldap.SCOPE_BASE,
+                                        attrlist=[self.member_attribute])
         except ldap.NO_SUCH_OBJECT:
-            return []
-        finally:
-            conn.unbind_s()
+            raise self.NotFound(group_id=group_id)
+
         users = []
         for dn, member in attrs:
             user_dns = member.get(self.member_attribute, [])
             for user_dn in user_dns:
-                if self.use_dumb_member and user_dn == self.dumb_member:
+                if self._is_dumb_member(user_dn):
                     continue
                 users.append(user_dn)
         return users
+
+    def get_filtered(self, group_id):
+        group = self.get(group_id)
+        return common_ldap.filter_entity(group)
+
+    def get_all_filtered(self, query=None):
+        return [common_ldap.filter_entity(group)
+                for group in self.get_all(query)]
