@@ -12,18 +12,19 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystone.common import dependency
 from keystone.common import sql
-from keystone.common.sql import migration_helpers
 from keystone.common import utils
+from keystone import config
 from keystone import exception
+from keystone.i18n import _
 from keystone import identity
-from keystone.openstack.common.db.sqlalchemy import migration
-from keystone.openstack.common.gettextutils import _
 
 # Import assignment sql to ensure that the models defined in there are
 # available for the reference from User and Group to Domain.id.
-from keystone.assignment.backends import sql as assignment_sql  # flake8: noqa
+from keystone.assignment.backends import sql as assignment_sql  # noqa
+
+
+CONF = config.CONF
 
 
 class User(sql.ModelBase, sql.DictBase):
@@ -74,16 +75,18 @@ class UserGroupMembership(sql.ModelBase, sql.DictBase):
                           primary_key=True)
 
 
-@dependency.requires('assignment_api')
 class Identity(identity.Driver):
+    # NOTE(henry-nash): Override the __init__() method so as to take a
+    # config parameter to enable sql to be used as a domain-specific driver.
+    def __init__(self, conf=None):
+        super(Identity, self).__init__()
+
     def default_assignment_driver(self):
         return "keystone.assignment.backends.sql.Assignment"
 
-    # Internal interface to manage the database
-    def db_sync(self, version=None):
-        migration.db_sync(
-            sql.get_engine(), migration_helpers.find_migrate_repo(),
-            version=version)
+    @property
+    def is_sql(self):
+        return True
 
     def _check_password(self, password, user_ref):
         """Check the specified password against the data store.
@@ -99,9 +102,6 @@ class Identity(identity.Driver):
         """
         return utils.check_password(password, user_ref.password)
 
-    def is_domain_aware(self):
-        return True
-
     # Identity interface
     def authenticate(self, user_id, password):
         session = sql.get_session()
@@ -156,8 +156,6 @@ class Identity(identity.Driver):
     @sql.handle_conflicts(conflict_type='user')
     def update_user(self, user_id, user):
         session = sql.get_session()
-        if 'id' in user and user_id != user['id']:
-            raise exception.ValidationError(_('Cannot change user ID'))
 
         with session.begin():
             user_ref = self._get_user(session, user_id)
@@ -195,7 +193,10 @@ class Identity(identity.Driver):
         query = query.filter_by(user_id=user_id)
         query = query.filter_by(group_id=group_id)
         if not query.first():
-            raise exception.NotFound(_('User not found in group'))
+            raise exception.NotFound(_("User '%(user_id)s' not found in"
+                                       " group '%(group_id)s'") %
+                                     {'user_id': user_id,
+                                      'group_id': group_id})
 
     def remove_user_from_group(self, user_id, group_id):
         session = sql.get_session()
@@ -206,7 +207,14 @@ class Identity(identity.Driver):
         query = query.filter_by(group_id=group_id)
         membership_ref = query.first()
         if membership_ref is None:
-            raise exception.NotFound(_('User not found in group'))
+            # Check if the group and user exist to return descriptive
+            # exceptions.
+            self.get_group(group_id)
+            self.get_user(user_id)
+            raise exception.NotFound(_("User '%(user_id)s' not found in"
+                                       " group '%(group_id)s'") %
+                                     {'user_id': user_id,
+                                      'group_id': group_id})
         with session.begin():
             session.delete(membership_ref)
 
@@ -246,7 +254,6 @@ class Identity(identity.Driver):
             q.delete(False)
 
             session.delete(ref)
-        self.assignment_api.delete_user(user_id)
 
     # group crud
 
@@ -302,4 +309,3 @@ class Identity(identity.Driver):
             q.delete(False)
 
             session.delete(ref)
-        self.assignment_api.delete_group(group_id)
