@@ -14,19 +14,15 @@
 
 import hashlib
 
+import six
+
 from keystone.identity import generator
 
 
 class Generator(generator.IDGenerator):
+
     def generate_public_ID(self, mapping):
         m = hashlib.sha256()
-        for key in sorted(mapping.keys()):
-            # python-ldap >3.0 returns bytes data type for attribute values
-            # except distinguished names, relative distinguished names,
-            # attribute names, queries on python3.
-            # Please see Bytes/text management in python-ldap module.
-            if isinstance(mapping[key], bytes):
-                m.update(mapping[key])
-            else:
-                m.update(mapping[key].encode('utf-8'))
+        for key in sorted(six.iterkeys(mapping)):
+            m.update(mapping[key])
         return m.hexdigest()
