@@ -16,6 +16,7 @@ from __future__ import absolute_import
 import atexit
 import copy
 import functools
+import logging
 import os
 import re
 import shutil
@@ -25,36 +26,31 @@ import time
 import warnings
 
 import fixtures
-import logging
-from paste import deploy
+from oslo.config import fixture as config_fixture
+import oslotest.base as oslotest
+from oslotest import mockpatch
 import six
-import testtools
 from testtools import testcase
 import webob
 
-from keystone.openstack.common.fixture import mockpatch
-from keystone.openstack.common import gettextutils
-
 # NOTE(ayoung)
 # environment.use_eventlet must run before any of the code that will
 # call the eventlet monkeypatching.
-from keystone.common import environment
+from keystone.common import environment  # noqa
 environment.use_eventlet()
 
 from keystone import auth
+from keystone import backends
+from keystone.common import config as common_cfg
 from keystone.common import dependency
 from keystone.common import kvs
 from keystone.common.kvs import core as kvs_core
-from keystone.common import sql
-from keystone.common.sql import migration_helpers
 from keystone.common import utils as common_utils
 from keystone import config
+from keystone import controllers
 from keystone import exception
+from keystone.i18n import _
 from keystone import notifications
-from keystone.openstack.common.db import options as db_options
-from keystone.openstack.common.db.sqlalchemy import migration
-from keystone.openstack.common.fixture import config as config_fixture
-from keystone.openstack.common.gettextutils import _
 from keystone.openstack.common import log
 from keystone import service
 from keystone.tests import ksfixtures
@@ -67,7 +63,6 @@ from keystone.openstack.common import policy as common_policy  # noqa
 
 config.configure()
 
-
 LOG = log.getLogger(__name__)
 PID = six.text_type(os.getpid())
 TESTSDIR = os.path.dirname(os.path.abspath(__file__))
@@ -88,6 +83,8 @@ TMPDIR = _calc_tmpdir()
 
 CONF = config.CONF
 
+IN_MEM_DB_CONN_STRING = 'sqlite://'
+
 exception._FATAL_EXCEPTION_FORMAT_ERRORS = True
 os.makedirs(TMPDIR)
 atexit.register(shutil.rmtree, TMPDIR)
@@ -119,19 +116,6 @@ class dirs:
 DEFAULT_TEST_DB_FILE = dirs.tmp('test.db')
 
 
-def _initialize_sql_session():
-    # Make sure the DB is located in the correct location, in this case set
-    # the default value, as this should be able to be overridden in some
-    # test cases.
-    db_file = DEFAULT_TEST_DB_FILE
-    db_options.set_defaults(
-        sql_connection='sqlite:///%s' % db_file,
-        sqlite_db=db_file)
-
-
-_initialize_sql_session()
-
-
 def checkout_vendor(repo, rev):
     # TODO(termie): this function is a good target for some optimizations :PERF
     name = repo.split('/')[-1]
@@ -164,25 +148,6 @@ def checkout_vendor(repo, rev):
     return revdir
 
 
-def setup_database():
-    db = dirs.tmp('test.db')
-    pristine = dirs.tmp('test.db.pristine')
-
-    if os.path.exists(db):
-        os.unlink(db)
-    if not os.path.exists(pristine):
-        migration.db_sync(sql.get_engine(),
-                          migration_helpers.find_migrate_repo())
-        migration_helpers.sync_database_to_version(extension='revoke')
-        shutil.copyfile(db, pristine)
-    else:
-        shutil.copyfile(pristine, db)
-
-
-def teardown_database():
-    sql.cleanup()
-
-
 @atexit.register
 def remove_test_databases():
     db = dirs.tmp('test.db')
@@ -250,6 +215,19 @@ def skip_if_cache_disabled(*sections):
     return wrapper
 
 
+def skip_if_no_multiple_domains_support(f):
+    """This decorator is used to skip a test if an identity driver
+    does not support multiple domains.
+    """
+    @functools.wraps(f)
+    def wrapper(*args, **kwargs):
+        test_obj = args[0]
+        if not test_obj.identity_api.multiple_domains_supported:
+                raise testcase.TestSkipped('No multiple domains support')
+        return f(*args, **kwargs)
+    return wrapper
+
+
 class UnexpectedExit(Exception):
     pass
 
@@ -284,7 +262,52 @@ class TestClient(object):
         return self.request('PUT', path=path, headers=headers, body=body)
 
 
-class BaseTestCase(testtools.TestCase):
+class NoModule(object):
+    """A mixin class to provide support for unloading/disabling modules."""
+
+    def setUp(self):
+        super(NoModule, self).setUp()
+
+        self._finders = []
+
+        def cleanup_finders():
+            for finder in self._finders:
+                sys.meta_path.remove(finder)
+            del self._finders
+        self.addCleanup(cleanup_finders)
+
+        self._cleared_modules = {}
+
+        def cleanup_modules():
+            sys.modules.update(self._cleared_modules)
+            del self._cleared_modules
+        self.addCleanup(cleanup_modules)
+
+    def clear_module(self, module):
+        cleared_modules = {}
+        for fullname in sys.modules.keys():
+            if fullname == module or fullname.startswith(module + '.'):
+                cleared_modules[fullname] = sys.modules.pop(fullname)
+        return cleared_modules
+
+    def disable_module(self, module):
+        """Ensure ImportError for the specified module."""
+
+        # Clear 'module' references in sys.modules
+        self._cleared_modules.update(self.clear_module(module))
+
+        # Disallow further imports of 'module'
+        class NoModule(object):
+            def find_module(self, fullname, path):
+                if fullname == module or fullname.startswith(module + '.'):
+                    raise ImportError
+
+        finder = NoModule()
+        self._finders.append(finder)
+        sys.meta_path.insert(0, finder)
+
+
+class BaseTestCase(oslotest.BaseTestCase):
     """Light weight base test class.
 
     This is a placeholder that will eventually go away once thc
@@ -318,14 +341,12 @@ class TestCase(BaseTestCase):
         return copy.copy(self._config_file_list)
 
     def config_overrides(self):
+        signing_certfile = 'examples/pki/certs/signing_cert.pem'
+        signing_keyfile = 'examples/pki/private/signing_key.pem'
+        # Exercise multiple worker process code paths
+        self.config_fixture.config(public_workers=2)
+        self.config_fixture.config(admin_workers=2)
         self.config_fixture.config(policy_file=dirs.etc('policy.json'))
-        self.config_fixture.config(
-            group='auth',
-            methods=['keystone.auth.plugins.external.DefaultDomain',
-                     'keystone.auth.plugins.password.Password',
-                     'keystone.auth.plugins.token.Token',
-                     'keystone.auth.plugins.oauth1.OAuth',
-                     'keystone.auth.plugins.saml2.Saml2'])
         self.config_fixture.config(
             # TODO(morganfainberg): Make Cache Testing a separate test case
             # in tempest, and move it out of the base unit tests.
@@ -339,7 +360,7 @@ class TestCase(BaseTestCase):
             template_file=dirs.tests('default_catalog.templates'))
         self.config_fixture.config(
             group='identity',
-            driver='keystone.identity.backends.kvs.Identity')
+            driver='keystone.identity.backends.sql.Identity')
         self.config_fixture.config(
             group='kvs',
             backends=[
@@ -349,16 +370,50 @@ class TestCase(BaseTestCase):
             group='revoke',
             driver='keystone.contrib.revoke.backends.kvs.Revoke')
         self.config_fixture.config(
-            group='signing',
-            certfile='examples/pki/certs/signing_cert.pem',
-            keyfile='examples/pki/private/signing_key.pem',
+            group='signing', certfile=signing_certfile,
+            keyfile=signing_keyfile,
             ca_certs='examples/pki/certs/cacert.pem')
         self.config_fixture.config(
             group='token',
-            driver='keystone.token.backends.kvs.Token')
+            driver='keystone.token.persistence.backends.kvs.Token')
         self.config_fixture.config(
             group='trust',
             driver='keystone.trust.backends.kvs.Trust')
+        self.config_fixture.config(
+            group='saml', certfile=signing_certfile, keyfile=signing_keyfile)
+        self.config_fixture.config(
+            default_log_levels=[
+                'amqp=WARN',
+                'amqplib=WARN',
+                'boto=WARN',
+                'qpid=WARN',
+                'sqlalchemy=WARN',
+                'suds=INFO',
+                'oslo.messaging=INFO',
+                'iso8601=WARN',
+                'requests.packages.urllib3.connectionpool=WARN',
+                'routes.middleware=INFO',
+                'stevedore.extension=INFO',
+                'keystone.notifications=INFO',
+                'keystone.common._memcache_pool=INFO',
+            ])
+        self.auth_plugin_config_override()
+
+    def auth_plugin_config_override(self, methods=None, **method_classes):
+        if methods is None:
+            methods = ['external', 'password', 'token', 'oauth1', 'saml2']
+            if not method_classes:
+                method_classes = dict(
+                    external='keystone.auth.plugins.external.DefaultDomain',
+                    password='keystone.auth.plugins.password.Password',
+                    token='keystone.auth.plugins.token.Token',
+                    oauth1='keystone.auth.plugins.oauth1.OAuth',
+                    saml2='keystone.auth.plugins.saml2.Saml2',
+                )
+        self.config_fixture.config(group='auth', methods=methods)
+        common_cfg.setup_authentication()
+        if method_classes:
+            self.config_fixture.config(group='auth', **method_classes)
 
     def setUp(self):
         super(TestCase, self).setUp()
@@ -388,9 +443,33 @@ class TestCase(BaseTestCase):
         self.config_fixture = self.useFixture(config_fixture.Config(CONF))
         self.config(self.config_files())
 
+        # NOTE(morganfainberg): mock the auth plugin setup to use the config
+        # fixture which automatically unregisters options when performing
+        # cleanup.
+        def mocked_register_auth_plugin_opt(conf, opt):
+            self.config_fixture.register_opt(opt, group='auth')
+        self.register_auth_plugin_opt_patch = self.useFixture(
+            mockpatch.PatchObject(common_cfg, '_register_auth_plugin_opt',
+                                  new=mocked_register_auth_plugin_opt))
+
         self.config_overrides()
 
         self.logger = self.useFixture(fixtures.FakeLogger(level=logging.DEBUG))
+
+        # NOTE(morganfainberg): This code is a copy from the oslo-incubator
+        # log module. This is not in a function or otherwise available to use
+        # without having a CONF object to setup logging. This should help to
+        # reduce the log size by limiting what we log (similar to how Keystone
+        # would run under mod_wsgi or eventlet).
+        for pair in CONF.default_log_levels:
+            mod, _sep, level_name = pair.partition('=')
+            logger = logging.getLogger(mod)
+            if sys.version_info < (2, 7):
+                level = logging.getLevelName(level_name)
+                logger.setLevel(level)
+            else:
+                logger.setLevel(level_name)
+
         warnings.filterwarnings('ignore', category=DeprecationWarning)
         self.useFixture(ksfixtures.Cache())
 
@@ -401,12 +480,14 @@ class TestCase(BaseTestCase):
         self.addCleanup(kvs.INMEMDB.clear)
 
         # Ensure Notification subscriotions and resource types are empty
-        self.addCleanup(notifications.SUBSCRIBERS.clear)
-        self.addCleanup(notifications._reset_notifier)
+        self.addCleanup(notifications.clear_subscribers)
+        self.addCleanup(notifications.reset_notifier)
 
         # Reset the auth-plugin registry
         self.addCleanup(self.clear_auth_plugin_registry)
 
+        self.addCleanup(setattr, controllers, '_VERSIONS', [])
+
     def config(self, config_files):
         CONF(args=[], project='keystone', default_config_files=config_files)
 
@@ -425,7 +506,7 @@ class TestCase(BaseTestCase):
         kvs_core.KEY_VALUE_STORE_REGISTRY.clear()
 
         self.clear_auth_plugin_registry()
-        drivers = service.load_backends()
+        drivers = backends.load_backends()
 
         drivers.update(dependency.resolve_future_dependencies())
 
@@ -433,15 +514,6 @@ class TestCase(BaseTestCase):
             setattr(self, manager_name, manager)
         self.addCleanup(self.cleanup_instance(*drivers.keys()))
 
-        # The credential backend only supports SQL, so we always have to load
-        # the tables.
-        self.engine = sql.get_engine()
-        self.addCleanup(sql.cleanup)
-        self.addCleanup(self.cleanup_instance('engine'))
-
-        sql.ModelBase.metadata.create_all(bind=self.engine)
-        self.addCleanup(sql.ModelBase.metadata.drop_all, bind=self.engine)
-
     def load_fixtures(self, fixtures):
         """Hacky basic and naive fixture loading based on a python module.
 
@@ -469,11 +541,15 @@ class TestCase(BaseTestCase):
                 fixtures_to_cleanup.append(attrname)
 
             for tenant in fixtures.TENANTS:
-                try:
-                    rv = self.assignment_api.create_project(
-                        tenant['id'], tenant)
-                except exception.Conflict:
-                    rv = self.assignment_api.get_project(tenant['id'])
+                if hasattr(self, 'tenant_%s' % tenant['id']):
+                    try:
+                        # This will clear out any roles on the project as well
+                        self.assignment_api.delete_project(tenant['id'])
+                    except exception.ProjectNotFound:
+                        pass
+                rv = self.assignment_api.create_project(
+                    tenant['id'], tenant)
+
                 attrname = 'tenant_%s' % tenant['id']
                 setattr(self, attrname, rv)
                 fixtures_to_cleanup.append(attrname)
@@ -491,15 +567,28 @@ class TestCase(BaseTestCase):
                 user_copy = user.copy()
                 tenants = user_copy.pop('tenants')
                 try:
-                    self.identity_api.create_user(user['id'], user_copy)
-                except exception.Conflict:
+                    existing_user = getattr(self, 'user_%s' % user['id'], None)
+                    if existing_user is not None:
+                        self.identity_api.delete_user(existing_user['id'])
+                except exception.UserNotFound:
                     pass
+
+                # For users, the manager layer will generate the ID
+                user_copy = self.identity_api.create_user(user_copy)
+                # Our tests expect that the password is still in the user
+                # record so that they can reference it, so put it back into
+                # the dict returned.
+                user_copy['password'] = user['password']
+
                 for tenant_id in tenants:
                     try:
-                        self.assignment_api.add_user_to_project(tenant_id,
-                                                                user['id'])
+                        self.assignment_api.add_user_to_project(
+                            tenant_id, user_copy['id'])
                     except exception.Conflict:
                         pass
+                # Use the ID from the fixture as the attribute name, so
+                # that our tests can easily reference each user dict, while
+                # the ID in the dict will be the real public ID.
                 attrname = 'user_%s' % user['id']
                 setattr(self, attrname, user_copy)
                 fixtures_to_cleanup.append(attrname)
@@ -516,7 +605,7 @@ class TestCase(BaseTestCase):
         return config
 
     def loadapp(self, config, name='main'):
-        return deploy.loadapp(self._paste_config(config), name=name)
+        return service.loadapp(self._paste_config(config), name=name)
 
     def client(self, app, *args, **kw):
         return TestClient(app, *args, **kw)
@@ -556,11 +645,11 @@ class TestCase(BaseTestCase):
             if isinstance(expected_regexp, six.string_types):
                 expected_regexp = re.compile(expected_regexp)
 
-            if isinstance(exc_value.args[0], gettextutils.Message):
-                if not expected_regexp.search(six.text_type(exc_value)):
+            if isinstance(exc_value.args[0], unicode):
+                if not expected_regexp.search(unicode(exc_value)):
                     raise self.failureException(
                         '"%s" does not match "%s"' %
-                        (expected_regexp.pattern, six.text_type(exc_value)))
+                        (expected_regexp.pattern, unicode(exc_value)))
             else:
                 if not expected_regexp.search(str(exc_value)):
                     raise self.failureException(
@@ -710,7 +799,7 @@ class SQLDriverOverrides(object):
             driver='keystone.contrib.revoke.backends.sql.Revoke')
         self.config_fixture.config(
             group='token',
-            driver='keystone.token.backends.sql.Token')
+            driver='keystone.token.persistence.backends.sql.Token')
         self.config_fixture.config(
             group='trust',
             driver='keystone.trust.backends.sql.Trust')
