@@ -14,17 +14,27 @@
 import datetime
 import uuid
 
+from oslo.utils import timeutils
 import six
+from testtools import matchers
 
+from keystone import config
 from keystone import exception
-from keystone.openstack.common import timeutils
 from keystone import tests
 from keystone.tests import default_fixtures
+from keystone.tests.ksfixtures import database
 from keystone.tests import test_backend
 
 
+CONF = config.CONF
+
+
 class KvsIdentity(tests.TestCase, test_backend.IdentityTests):
     def setUp(self):
+        # NOTE(dstanek): setup the database for subsystems that only have a
+        # SQL backend (like credentials)
+        self.useFixture(database.Database())
+
         super(KvsIdentity, self).setUp()
         self.load_backends()
         self.load_fixtures(default_fixtures)
@@ -35,6 +45,12 @@ class KvsIdentity(tests.TestCase, test_backend.IdentityTests):
             group='identity',
             driver='keystone.identity.backends.kvs.Identity')
 
+    def test_password_hashed(self):
+        driver = self.identity_api._select_identity_driver(
+            self.user_foo['domain_id'])
+        user_ref = driver._get_user(self.user_foo['id'])
+        self.assertNotEqual(user_ref['password'], self.user_foo['password'])
+
     def test_list_projects_for_user_with_grants(self):
         self.skipTest('kvs backend is now deprecated')
 
@@ -65,6 +81,49 @@ class KvsIdentity(tests.TestCase, test_backend.IdentityTests):
     def test_move_project_between_domains_with_clashing_names_fails(self):
         self.skipTest('Blocked by bug 1119770')
 
+    def test_delete_group_removes_role_assignments(self):
+        # When a group is deleted any role assignments for the group are
+        # supposed to be removed, but the KVS backend doesn't implement the
+        # funcationality so the assignments are left around.
+
+        DEFAULT_DOMAIN_ID = CONF.identity.default_domain_id
+        MEMBER_ROLE_ID = 'member'
+
+        def get_member_assignments():
+            assignments = self.assignment_api.list_role_assignments()
+            return filter(lambda x: x['role_id'] == MEMBER_ROLE_ID,
+                          assignments)
+
+        # Create a group.
+        new_group = {
+            'domain_id': DEFAULT_DOMAIN_ID,
+            'name': self.getUniqueString(prefix='tdgrra')}
+        new_group = self.identity_api.create_group(new_group)
+
+        # Create a project.
+        new_project = {
+            'id': uuid.uuid4().hex,
+            'name': self.getUniqueString(prefix='tdgrra'),
+            'domain_id': DEFAULT_DOMAIN_ID}
+        self.assignment_api.create_project(new_project['id'], new_project)
+
+        # Assign a role to the group.
+        self.assignment_api.create_grant(
+            group_id=new_group['id'], project_id=new_project['id'],
+            role_id=MEMBER_ROLE_ID)
+
+        new_role_assignments = get_member_assignments()
+
+        # Delete the group.
+        self.identity_api.delete_group(new_group['id'])
+
+        # Check that the role assignment for the group is still there since
+        # kvs doesn't implement cleanup.
+        member_assignments = get_member_assignments()
+
+        self.assertThat(member_assignments,
+                        matchers.Equals(new_role_assignments))
+
 
 class KvsToken(tests.TestCase, test_backend.TokenTests):
     def setUp(self):
@@ -78,11 +137,13 @@ class KvsToken(tests.TestCase, test_backend.TokenTests):
             driver='keystone.identity.backends.kvs.Identity')
 
     def test_flush_expired_token(self):
-        self.assertRaises(exception.NotImplemented,
-                          self.token_api.flush_expired_tokens)
+        self.assertRaises(
+            exception.NotImplemented,
+            self.token_provider_api._persistence.flush_expired_tokens)
 
     def _update_user_token_index_direct(self, user_key, token_id, new_data):
-        token_list = self.token_api.driver._get_user_token_list_with_expiry(
+        persistence = self.token_provider_api._persistence
+        token_list = persistence.driver._get_user_token_list_with_expiry(
             user_key)
         # Update the user-index so that the expires time is _actually_ expired
         # since we do not do an explicit get on the token, we only reference
@@ -92,7 +153,8 @@ class KvsToken(tests.TestCase, test_backend.TokenTests):
             if data[0] == token_id:
                 token_list[i] = new_data
                 break
-        self.token_api.driver._store.set(user_key, token_list)
+        self.token_provider_api._persistence.driver._store.set(user_key,
+                                                               token_list)
 
     def test_cleanup_user_index_on_create(self):
         user_id = six.text_type(uuid.uuid4().hex)
@@ -103,12 +165,12 @@ class KvsToken(tests.TestCase, test_backend.TokenTests):
         expire_delta = datetime.timedelta(seconds=86400)
 
         # NOTE(morganfainberg): Directly access the data cache since we need to
-        # get expired tokens as well as valid tokens. token_api.list_tokens()
-        # will not return any expired tokens in the list.
-        user_key = self.token_api.driver._prefix_user_id(user_id)
-        user_token_list = self.token_api.driver._store.get(user_key)
-        valid_token_ref = self.token_api.get_token(valid_token_id)
-        expired_token_ref = self.token_api.get_token(expired_token_id)
+        # get expired tokens as well as valid tokens.
+        token_persistence = self.token_provider_api._persistence
+        user_key = token_persistence.driver._prefix_user_id(user_id)
+        user_token_list = token_persistence.driver._store.get(user_key)
+        valid_token_ref = token_persistence.get_token(valid_token_id)
+        expired_token_ref = token_persistence.get_token(expired_token_id)
         expected_user_token_list = [
             (valid_token_id, timeutils.isotime(valid_token_ref['expires'],
                                                subsecond=True)),
@@ -123,25 +185,25 @@ class KvsToken(tests.TestCase, test_backend.TokenTests):
                                              new_expired_data)
         valid_token_id_2, valid_data_2 = self.create_token_sample_data(
             user_id=user_id)
-        valid_token_ref_2 = self.token_api.get_token(valid_token_id_2)
+        valid_token_ref_2 = token_persistence.get_token(valid_token_id_2)
         expected_user_token_list = [
             (valid_token_id, timeutils.isotime(valid_token_ref['expires'],
                                                subsecond=True)),
             (valid_token_id_2, timeutils.isotime(valid_token_ref_2['expires'],
                                                  subsecond=True))]
-        user_token_list = self.token_api.driver._store.get(user_key)
+        user_token_list = token_persistence.driver._store.get(user_key)
         self.assertEqual(expected_user_token_list, user_token_list)
 
         # Test that revoked tokens are removed from the list on create.
-        self.token_api.delete_token(valid_token_id_2)
+        token_persistence.delete_token(valid_token_id_2)
         new_token_id, data = self.create_token_sample_data(user_id=user_id)
-        new_token_ref = self.token_api.get_token(new_token_id)
+        new_token_ref = token_persistence.get_token(new_token_id)
         expected_user_token_list = [
             (valid_token_id, timeutils.isotime(valid_token_ref['expires'],
                                                subsecond=True)),
             (new_token_id, timeutils.isotime(new_token_ref['expires'],
                                              subsecond=True))]
-        user_token_list = self.token_api.driver._store.get(user_key)
+        user_token_list = token_persistence.driver._store.get(user_key)
         self.assertEqual(expected_user_token_list, user_token_list)
 
 
@@ -219,6 +281,12 @@ class KvsCatalog(tests.TestCase, test_backend.CatalogTests):
         f = super(KvsCatalog, self).test_get_v3_catalog_endpoint_disabled
         self.assertRaises(exception.NotFound, f)
 
+    def test_list_regions_filtered_by_parent_region_id(self):
+        self.skipTest('KVS backend does not support hints')
+
+    def test_service_filtering(self):
+        self.skipTest("kvs backend doesn't support filtering")
+
 
 class KvsTokenCacheInvalidation(tests.TestCase,
                                 test_backend.TokenCacheInvalidation):
@@ -235,3 +303,23 @@ class KvsTokenCacheInvalidation(tests.TestCase,
         self.config_fixture.config(
             group='token',
             driver='keystone.token.backends.kvs.Token')
+
+
+class KvsInheritanceTests(tests.TestCase, test_backend.InheritanceTests):
+    def setUp(self):
+        # NOTE(dstanek): setup the database for subsystems that only have a
+        # SQL backend (like credentials)
+        self.useFixture(database.Database())
+
+        super(KvsInheritanceTests, self).setUp()
+        self.load_backends()
+        self.load_fixtures(default_fixtures)
+
+    def config_overrides(self):
+        super(KvsInheritanceTests, self).config_overrides()
+        self.config_fixture.config(
+            group='identity',
+            driver='keystone.identity.backends.kvs.Identity')
+        self.config_fixture.config(
+            group='assignment',
+            driver='keystone.assignment.backends.kvs.Assignment')
