@@ -13,35 +13,39 @@
 # under the License.
 
 import datetime
+import os
 import uuid
-import webob
 
-from keystoneclient import exceptions as client_exceptions
-from keystoneclient.v2_0 import client as ks_client
 import mock
+from oslo.utils import timeutils
+import webob
 
 from keystone import config
 from keystone.openstack.common import jsonutils
-from keystone.openstack.common import timeutils
 from keystone import tests
 from keystone.tests import default_fixtures
 from keystone.tests.ksfixtures import appserver
+from keystone.tests.ksfixtures import database
 
 
 CONF = config.CONF
 DEFAULT_DOMAIN_ID = CONF.identity.default_domain_id
+OPENSTACK_REPO = os.environ.get('OPENSTACK_REPO',
+                                'https://git.openstack.org/openstack')
+KEYSTONECLIENT_REPO = '%s/python-keystoneclient.git' % OPENSTACK_REPO
 
 
-class ClientDrivenTestCase(tests.TestCase):
+class CompatTestCase(tests.NoModule, tests.TestCase):
 
     def setUp(self):
-        super(ClientDrivenTestCase, self).setUp()
+        super(CompatTestCase, self).setUp()
 
         # FIXME(morganfainberg): Since we are running tests through the
         # controllers and some internal api drivers are SQL-only, the correct
         # approach is to ensure we have the correct backing store. The
         # credential api makes some very SQL specific assumptions that should
         # be addressed allowing for non-SQL based testing to occur.
+        self.useFixture(database.Database())
         self.load_backends()
 
         self.load_fixtures(default_fixtures)
@@ -61,6 +65,13 @@ class ClientDrivenTestCase(tests.TestCase):
 
         self.addCleanup(self.cleanup_instance('public_server', 'admin_server'))
 
+        if isinstance(self.checkout_info, str):
+            revdir = self.checkout_info
+        else:
+            revdir = tests.checkout_vendor(*self.checkout_info)
+        self.add_path(revdir)
+        self.clear_module('keystoneclient')
+
     def _public_url(self):
         public_port = self.public_server.socket_info['socket'][1]
         return "http://localhost:%s/v2.0" % public_port
@@ -70,6 +81,8 @@ class ClientDrivenTestCase(tests.TestCase):
         return "http://localhost:%s/v2.0" % admin_port
 
     def _client(self, admin=False, **kwargs):
+        from keystoneclient.v2_0 import client as ks_client
+
         url = self._admin_url() if admin else self._public_url()
         kc = ks_client.Client(endpoint=url,
                               auth_url=self._public_url(),
@@ -84,7 +97,14 @@ class ClientDrivenTestCase(tests.TestCase):
             user_ref = self.user_foo
         if tenant_ref is None:
             for user in default_fixtures.USERS:
-                if user['id'] == user_ref['id']:
+                # The fixture ID is no longer used as the ID in the database
+                # The fixture ID, however, is still used as part of the
+                # attribute name when storing the created object on the test
+                # case. This means that we need to use the fixture ID below to
+                # find the actial object so that we can get the ID as stored
+                # in the database to compare against.
+                if (getattr(self, 'user_%s' % user['id'])['id'] ==
+                        user_ref['id']):
                     tenant_id = user['tenants'][0]
         else:
             tenant_id = tenant_ref['id']
@@ -94,19 +114,24 @@ class ClientDrivenTestCase(tests.TestCase):
                             tenant_id=tenant_id,
                             admin=admin)
 
+
+class KeystoneClientTests(object):
+    """Tests for all versions of keystoneclient."""
+
     def test_authenticate_tenant_name_and_tenants(self):
         client = self.get_client()
         tenants = client.tenants.list()
-        self.assertEqual(tenants[0].id, self.tenant_bar['id'])
+        self.assertEqual(self.tenant_bar['id'], tenants[0].id)
 
     def test_authenticate_tenant_id_and_tenants(self):
         client = self._client(username=self.user_foo['name'],
                               password=self.user_foo['password'],
                               tenant_id='bar')
         tenants = client.tenants.list()
-        self.assertEqual(tenants[0].id, self.tenant_bar['id'])
+        self.assertEqual(self.tenant_bar['id'], tenants[0].id)
 
     def test_authenticate_invalid_tenant_id(self):
+        from keystoneclient import exceptions as client_exceptions
         self.assertRaises(client_exceptions.Unauthorized,
                           self._client,
                           username=self.user_foo['name'],
@@ -118,16 +143,17 @@ class ClientDrivenTestCase(tests.TestCase):
         token = client.auth_token
         token_client = self._client(token=token)
         tenants = token_client.tenants.list()
-        self.assertEqual(tenants[0].id, self.tenant_bar['id'])
+        self.assertEqual(self.tenant_bar['id'], tenants[0].id)
 
     def test_authenticate_token_tenant_id(self):
         client = self.get_client()
         token = client.auth_token
         token_client = self._client(token=token, tenant_id='bar')
         tenants = token_client.tenants.list()
-        self.assertEqual(tenants[0].id, self.tenant_bar['id'])
+        self.assertEqual(self.tenant_bar['id'], tenants[0].id)
 
     def test_authenticate_token_invalid_tenant_id(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client()
         token = client.auth_token
         self.assertRaises(client_exceptions.Unauthorized,
@@ -135,6 +161,7 @@ class ClientDrivenTestCase(tests.TestCase):
                           tenant_id=uuid.uuid4().hex)
 
     def test_authenticate_token_invalid_tenant_name(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client()
         token = client.auth_token
         self.assertRaises(client_exceptions.Unauthorized,
@@ -146,15 +173,17 @@ class ClientDrivenTestCase(tests.TestCase):
         token = client.auth_token
         token_client = self._client(token=token, tenant_name='BAR')
         tenants = token_client.tenants.list()
-        self.assertEqual(tenants[0].id, self.tenant_bar['id'])
-        self.assertEqual(tenants[0].id, self.tenant_bar['id'])
+        self.assertEqual(self.tenant_bar['id'], tenants[0].id)
+        self.assertEqual(self.tenant_bar['id'], tenants[0].id)
 
     def test_authenticate_and_delete_token(self):
+        from keystoneclient import exceptions as client_exceptions
+
         client = self.get_client(admin=True)
         token = client.auth_token
         token_client = self._client(token=token)
         tenants = token_client.tenants.list()
-        self.assertEqual(tenants[0].id, self.tenant_bar['id'])
+        self.assertEqual(self.tenant_bar['id'], tenants[0].id)
 
         client.tokens.delete(token_client.auth_token)
 
@@ -162,6 +191,8 @@ class ClientDrivenTestCase(tests.TestCase):
                           token_client.tenants.list)
 
     def test_authenticate_no_password(self):
+        from keystoneclient import exceptions as client_exceptions
+
         user_ref = self.user_foo.copy()
         user_ref['password'] = None
         self.assertRaises(client_exceptions.AuthorizationFailure,
@@ -169,6 +200,8 @@ class ClientDrivenTestCase(tests.TestCase):
                           user_ref)
 
     def test_authenticate_no_username(self):
+        from keystoneclient import exceptions as client_exceptions
+
         user_ref = self.user_foo.copy()
         user_ref['name'] = None
         self.assertRaises(client_exceptions.AuthorizationFailure,
@@ -176,6 +209,8 @@ class ClientDrivenTestCase(tests.TestCase):
                           user_ref)
 
     def test_authenticate_disabled_tenant(self):
+        from keystoneclient import exceptions as client_exceptions
+
         admin_client = self.get_client(admin=True)
 
         tenant = {
@@ -226,6 +261,8 @@ class ClientDrivenTestCase(tests.TestCase):
     # FIXME(ja): add a test that admin endpoint returns unauthorized if not
     #            admin
     def test_tenant_create_update_and_delete(self):
+        from keystoneclient import exceptions as client_exceptions
+
         tenant_name = 'original_tenant'
         tenant_description = 'My original tenant!'
         tenant_enabled = True
@@ -235,19 +272,19 @@ class ClientDrivenTestCase(tests.TestCase):
         tenant = client.tenants.create(tenant_name=tenant_name,
                                        description=tenant_description,
                                        enabled=tenant_enabled)
-        self.assertEqual(tenant.name, tenant_name)
-        self.assertEqual(tenant.description, tenant_description)
-        self.assertEqual(tenant.enabled, tenant_enabled)
+        self.assertEqual(tenant_name, tenant.name)
+        self.assertEqual(tenant_description, tenant.description)
+        self.assertEqual(tenant_enabled, tenant.enabled)
 
         tenant = client.tenants.get(tenant_id=tenant.id)
-        self.assertEqual(tenant.name, tenant_name)
-        self.assertEqual(tenant.description, tenant_description)
-        self.assertEqual(tenant.enabled, tenant_enabled)
+        self.assertEqual(tenant_name, tenant.name)
+        self.assertEqual(tenant_description, tenant.description)
+        self.assertEqual(tenant_enabled, tenant.enabled)
 
         tenant = [t for t in client.tenants.list() if t.id == tenant.id].pop()
-        self.assertEqual(tenant.name, tenant_name)
-        self.assertEqual(tenant.description, tenant_description)
-        self.assertEqual(tenant.enabled, tenant_enabled)
+        self.assertEqual(tenant_name, tenant.name)
+        self.assertEqual(tenant_description, tenant.description)
+        self.assertEqual(tenant_enabled, tenant.enabled)
 
         # update, get, and list a tenant
         tenant_name = 'updated_tenant'
@@ -257,19 +294,19 @@ class ClientDrivenTestCase(tests.TestCase):
                                        tenant_name=tenant_name,
                                        enabled=tenant_enabled,
                                        description=tenant_description)
-        self.assertEqual(tenant.name, tenant_name)
-        self.assertEqual(tenant.description, tenant_description)
-        self.assertEqual(tenant.enabled, tenant_enabled)
+        self.assertEqual(tenant_name, tenant.name)
+        self.assertEqual(tenant_description, tenant.description)
+        self.assertEqual(tenant_enabled, tenant.enabled)
 
         tenant = client.tenants.get(tenant_id=tenant.id)
-        self.assertEqual(tenant.name, tenant_name)
-        self.assertEqual(tenant.description, tenant_description)
-        self.assertEqual(tenant.enabled, tenant_enabled)
+        self.assertEqual(tenant_name, tenant.name)
+        self.assertEqual(tenant_description, tenant.description)
+        self.assertEqual(tenant_enabled, tenant.enabled)
 
         tenant = [t for t in client.tenants.list() if t.id == tenant.id].pop()
-        self.assertEqual(tenant.name, tenant_name)
-        self.assertEqual(tenant.description, tenant_description)
-        self.assertEqual(tenant.enabled, tenant_enabled)
+        self.assertEqual(tenant_name, tenant.name)
+        self.assertEqual(tenant_description, tenant.description)
+        self.assertEqual(tenant_enabled, tenant.enabled)
 
         # delete, get, and list a tenant
         client.tenants.delete(tenant=tenant.id)
@@ -279,6 +316,8 @@ class ClientDrivenTestCase(tests.TestCase):
                          if t.id == tenant.id])
 
     def test_tenant_create_update_and_delete_unicode(self):
+        from keystoneclient import exceptions as client_exceptions
+
         tenant_name = u'original \u540d\u5b57'
         tenant_description = 'My original tenant!'
         tenant_enabled = True
@@ -288,19 +327,19 @@ class ClientDrivenTestCase(tests.TestCase):
         tenant = client.tenants.create(tenant_name,
                                        description=tenant_description,
                                        enabled=tenant_enabled)
-        self.assertEqual(tenant.name, tenant_name)
-        self.assertEqual(tenant.description, tenant_description)
+        self.assertEqual(tenant_name, tenant.name)
+        self.assertEqual(tenant_description, tenant.description)
         self.assertIs(tenant.enabled, tenant_enabled)
 
         tenant = client.tenants.get(tenant.id)
-        self.assertEqual(tenant.name, tenant_name)
-        self.assertEqual(tenant.description, tenant_description)
+        self.assertEqual(tenant_name, tenant.name)
+        self.assertEqual(tenant_description, tenant.description)
         self.assertIs(tenant.enabled, tenant_enabled)
 
         # multiple tenants exist due to fixtures, so find the one we're testing
         tenant = [t for t in client.tenants.list() if t.id == tenant.id].pop()
-        self.assertEqual(tenant.name, tenant_name)
-        self.assertEqual(tenant.description, tenant_description)
+        self.assertEqual(tenant_name, tenant.name)
+        self.assertEqual(tenant_description, tenant.description)
         self.assertIs(tenant.enabled, tenant_enabled)
 
         # update, get, and list a tenant
@@ -311,18 +350,18 @@ class ClientDrivenTestCase(tests.TestCase):
                                        tenant_name=tenant_name,
                                        enabled=tenant_enabled,
                                        description=tenant_description)
-        self.assertEqual(tenant.name, tenant_name)
-        self.assertEqual(tenant.description, tenant_description)
+        self.assertEqual(tenant_name, tenant.name)
+        self.assertEqual(tenant_description, tenant.description)
         self.assertIs(tenant.enabled, tenant_enabled)
 
         tenant = client.tenants.get(tenant.id)
-        self.assertEqual(tenant.name, tenant_name)
-        self.assertEqual(tenant.description, tenant_description)
+        self.assertEqual(tenant_name, tenant.name)
+        self.assertEqual(tenant_description, tenant.description)
         self.assertIs(tenant.enabled, tenant_enabled)
 
         tenant = [t for t in client.tenants.list() if t.id == tenant.id].pop()
-        self.assertEqual(tenant.name, tenant_name)
-        self.assertEqual(tenant.description, tenant_description)
+        self.assertEqual(tenant_name, tenant.name)
+        self.assertEqual(tenant_description, tenant.description)
         self.assertIs(tenant.enabled, tenant_enabled)
 
         # delete, get, and list a tenant
@@ -333,24 +372,28 @@ class ClientDrivenTestCase(tests.TestCase):
                          if t.id == tenant.id])
 
     def test_tenant_create_no_name(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.BadRequest,
                           client.tenants.create,
                           tenant_name="")
 
     def test_tenant_delete_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.tenants.delete,
                           tenant=uuid.uuid4().hex)
 
     def test_tenant_get_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.tenants.get,
                           tenant_id=uuid.uuid4().hex)
 
     def test_tenant_update_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.tenants.update,
@@ -359,14 +402,16 @@ class ClientDrivenTestCase(tests.TestCase):
     def test_tenant_list(self):
         client = self.get_client()
         tenants = client.tenants.list()
-        self.assertEqual(len(tenants), 1)
+        self.assertEqual(1, len(tenants))
 
         # Admin endpoint should return *all* tenants
         client = self.get_client(admin=True)
         tenants = client.tenants.list()
-        self.assertEqual(len(tenants), len(default_fixtures.TENANTS))
+        self.assertEqual(len(default_fixtures.TENANTS), len(tenants))
 
     def test_invalid_password(self):
+        from keystoneclient import exceptions as client_exceptions
+
         good_client = self._client(username=self.user_foo['name'],
                                    password=self.user_foo['password'])
         good_client.tenants.list()
@@ -377,72 +422,39 @@ class ClientDrivenTestCase(tests.TestCase):
                           password=uuid.uuid4().hex)
 
     def test_invalid_user_and_password(self):
+        from keystoneclient import exceptions as client_exceptions
+
         self.assertRaises(client_exceptions.Unauthorized,
                           self._client,
                           username=uuid.uuid4().hex,
                           password=uuid.uuid4().hex)
 
     def test_change_password_invalidates_token(self):
-        admin_client = self.get_client(admin=True)
-
-        username = uuid.uuid4().hex
-        password = uuid.uuid4().hex
-        user = admin_client.users.create(name=username, password=password,
-                                         email=uuid.uuid4().hex)
-
-        # auth as user should work before a password change
-        client = self._client(username=username, password=password)
-
-        # auth as user with a token should work before a password change
-        self._client(token=client.auth_token)
-
-        # administrative password reset
-        admin_client.users.update_password(
-            user=user.id,
-            password=uuid.uuid4().hex)
-
-        # auth as user with original password should not work after change
-        self.assertRaises(client_exceptions.Unauthorized,
-                          self._client,
-                          username=username,
-                          password=password)
-
-        # authenticate with an old token should not work after change
-        self.assertRaises(client_exceptions.Unauthorized,
-                          self._client,
-                          token=client.auth_token)
-
-    def test_user_change_own_password_invalidates_token(self):
         from keystoneclient import exceptions as client_exceptions
 
-        # bootstrap a user as admin
         client = self.get_client(admin=True)
+
         username = uuid.uuid4().hex
-        password = uuid.uuid4().hex
-        client.users.create(name=username, password=password,
-                            email=uuid.uuid4().hex)
+        passwd = uuid.uuid4().hex
+        user = client.users.create(name=username, password=passwd,
+                                   email=uuid.uuid4().hex)
 
-        # auth as user should work before a password change
-        client = self._client(username=username, password=password)
+        token_id = client.tokens.authenticate(username=username,
+                                              password=passwd).id
 
-        # auth as user with a token should work before a password change
-        self._client(token=client.auth_token)
+        # authenticate with a token should work before a password change
+        client.tokens.authenticate(token=token_id)
 
-        # change the user's own password
-        client.users.update_own_password(password, uuid.uuid4().hex)
+        client.users.update_password(user=user.id, password=uuid.uuid4().hex)
 
-        # auth as user with original password should not work after change
+        # authenticate with a token should not work after a password change
         self.assertRaises(client_exceptions.Unauthorized,
-                          self._client,
-                          username=username,
-                          password=password)
-
-        # auth as user with an old token should not work after change
-        self.assertRaises(client_exceptions.Unauthorized,
-                          self._client,
-                          token=client.auth_token)
+                          client.tokens.authenticate,
+                          token=token_id)
 
     def test_disable_tenant_invalidates_token(self):
+        from keystoneclient import exceptions as client_exceptions
+
         admin_client = self.get_client(admin=True)
         foo_client = self.get_client(self.user_foo)
         tenant_bar = admin_client.tenants.get(self.tenant_bar['id'])
@@ -461,6 +473,8 @@ class ClientDrivenTestCase(tests.TestCase):
                           self.user_foo)
 
     def test_delete_tenant_invalidates_token(self):
+        from keystoneclient import exceptions as client_exceptions
+
         admin_client = self.get_client(admin=True)
         foo_client = self.get_client(self.user_foo)
         tenant_bar = admin_client.tenants.get(self.tenant_bar['id'])
@@ -479,6 +493,8 @@ class ClientDrivenTestCase(tests.TestCase):
                           self.user_foo)
 
     def test_disable_user_invalidates_token(self):
+        from keystoneclient import exceptions as client_exceptions
+
         admin_client = self.get_client(admin=True)
         foo_client = self.get_client(self.user_foo)
 
@@ -494,6 +510,8 @@ class ClientDrivenTestCase(tests.TestCase):
                           self.user_foo)
 
     def test_delete_user_invalidates_token(self):
+        from keystoneclient import exceptions as client_exceptions
+
         admin_client = self.get_client(admin=True)
         client = self.get_client(admin=False)
 
@@ -531,20 +549,22 @@ class ClientDrivenTestCase(tests.TestCase):
             timeutils.parse_isotime(reauthenticated_token.expires))
 
     def test_user_create_update_delete(self):
+        from keystoneclient import exceptions as client_exceptions
+
         test_username = 'new_user'
         client = self.get_client(admin=True)
         user = client.users.create(name=test_username,
                                    password='password',
                                    email='user1@test.com')
-        self.assertEqual(user.name, test_username)
+        self.assertEqual(test_username, user.name)
 
         user = client.users.get(user=user.id)
-        self.assertEqual(user.name, test_username)
+        self.assertEqual(test_username, user.name)
 
         user = client.users.update(user=user,
                                    name=test_username,
                                    email='user2@test.com')
-        self.assertEqual(user.email, 'user2@test.com')
+        self.assertEqual('user2@test.com', user.email)
 
         # NOTE(termie): update_enabled doesn't return anything, probably a bug
         client.users.update_enabled(user=user, enabled=False)
@@ -576,7 +596,7 @@ class ClientDrivenTestCase(tests.TestCase):
                                     password='password',
                                     email='user1@test.com',
                                     tenant_id='bar')
-        self.assertEqual(user2.name, test_username)
+        self.assertEqual(test_username, user2.name)
 
     def test_update_default_tenant_to_existing_value(self):
         client = self.get_client(admin=True)
@@ -592,6 +612,7 @@ class ClientDrivenTestCase(tests.TestCase):
             user=user, tenant=self.tenant_bar['id'])
 
     def test_user_create_no_string_password(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.BadRequest,
                           client.users.create,
@@ -600,6 +621,7 @@ class ClientDrivenTestCase(tests.TestCase):
                           email=uuid.uuid4().hex)
 
     def test_user_create_no_name(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.BadRequest,
                           client.users.create,
@@ -608,6 +630,7 @@ class ClientDrivenTestCase(tests.TestCase):
                           email=uuid.uuid4().hex)
 
     def test_user_create_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.users.create,
@@ -617,18 +640,21 @@ class ClientDrivenTestCase(tests.TestCase):
                           tenant_id=uuid.uuid4().hex)
 
     def test_user_get_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.users.get,
                           user=uuid.uuid4().hex)
 
     def test_user_list_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.users.list,
                           tenant_id=uuid.uuid4().hex)
 
     def test_user_update_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.users.update,
@@ -642,6 +668,7 @@ class ClientDrivenTestCase(tests.TestCase):
         self.assertEqual(tenant_id, user.tenant_id)
 
     def test_user_update_password_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.users.update_password,
@@ -649,6 +676,7 @@ class ClientDrivenTestCase(tests.TestCase):
                           password=uuid.uuid4().hex)
 
     def test_user_delete_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.users.delete,
@@ -669,16 +697,18 @@ class ClientDrivenTestCase(tests.TestCase):
     def test_role_get(self):
         client = self.get_client(admin=True)
         role = client.roles.get(role=self.role_admin['id'])
-        self.assertEqual(role.id, self.role_admin['id'])
+        self.assertEqual(self.role_admin['id'], role.id)
 
     def test_role_crud(self):
+        from keystoneclient import exceptions as client_exceptions
+
         test_role = 'new_role'
         client = self.get_client(admin=True)
         role = client.roles.create(name=test_role)
-        self.assertEqual(role.name, test_role)
+        self.assertEqual(test_role, role.name)
 
         role = client.roles.get(role=role.id)
-        self.assertEqual(role.name, test_role)
+        self.assertEqual(test_role, role.name)
 
         client.roles.delete(role=role.id)
 
@@ -690,24 +720,28 @@ class ClientDrivenTestCase(tests.TestCase):
                           role=role.id)
 
     def test_role_create_no_name(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.BadRequest,
                           client.roles.create,
                           name="")
 
     def test_role_get_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.roles.get,
                           role=uuid.uuid4().hex)
 
     def test_role_delete_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.roles.delete,
                           role=uuid.uuid4().hex)
 
     def test_role_list_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.roles.roles_for_user,
@@ -729,6 +763,7 @@ class ClientDrivenTestCase(tests.TestCase):
         self.assertTrue(len(roles) > 0)
 
     def test_service_crud(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
 
         service_name = uuid.uuid4().hex
@@ -761,27 +796,31 @@ class ClientDrivenTestCase(tests.TestCase):
                           client.services.get,
                           id=service.id)
         services = [x for x in client.services.list() if x.id == service.id]
-        self.assertEqual(len(services), 0)
+        self.assertEqual(0, len(services))
 
     def test_service_delete_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.services.delete,
                           id=uuid.uuid4().hex)
 
     def test_service_get_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.services.get,
                           id=uuid.uuid4().hex)
 
     def test_endpoint_delete_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.endpoints.delete,
                           id=uuid.uuid4().hex)
 
     def test_admin_requires_adminness(self):
+        from keystoneclient import exceptions as client_exceptions
         # FIXME(ja): this should be Unauthorized
         exception = client_exceptions.ClientException
 
@@ -839,7 +878,7 @@ class ClientDrivenTestCase(tests.TestCase):
                                    user=self.user_two['id'],
                                    role=self.role_other['id'])
         user_refs = client.tenants.list_users(tenant=self.tenant_bar['id'])
-        self.assertTrue(self.user_two['id'] in [x.id for x in user_refs])
+        self.assertIn(self.user_two['id'], [x.id for x in user_refs])
         client.roles.remove_user_role(tenant=self.tenant_bar['id'],
                                       user=self.user_two['id'],
                                       role=self.role_other['id'])
@@ -850,6 +889,7 @@ class ClientDrivenTestCase(tests.TestCase):
         self.assertNotIn(self.user_two['id'], [x.id for x in user_refs])
 
     def test_user_role_add_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.roles.add_user_role,
@@ -870,6 +910,7 @@ class ClientDrivenTestCase(tests.TestCase):
                                    role=self.role_member['id'])
 
     def test_user_role_remove_404(self):
+        from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
         self.assertRaises(client_exceptions.NotFound,
                           client.roles.remove_user_role,
@@ -905,14 +946,16 @@ class ClientDrivenTestCase(tests.TestCase):
                                                     self.user_foo['id'])
 
         tenants = client.tenants.list()
-        self.assertEqual(len(tenants), 3)
+        self.assertEqual(3, len(tenants))
 
         tenants_marker = client.tenants.list(marker=tenants[0].id)
-        self.assertEqual(len(tenants_marker), 2)
-        self.assertEqual(tenants[1].name, tenants_marker[0].name)
-        self.assertEqual(tenants[2].name, tenants_marker[1].name)
+        self.assertEqual(2, len(tenants_marker))
+        self.assertEqual(tenants_marker[0].name, tenants[1].name)
+        self.assertEqual(tenants_marker[1].name, tenants[2].name)
 
     def test_tenant_list_marker_not_found(self):
+        from keystoneclient import exceptions as client_exceptions
+
         client = self.get_client()
         self.assertRaises(client_exceptions.BadRequest,
                           client.tenants.list, marker=uuid.uuid4().hex)
@@ -930,14 +973,16 @@ class ClientDrivenTestCase(tests.TestCase):
                                                     self.user_foo['id'])
 
         tenants = client.tenants.list()
-        self.assertEqual(len(tenants), 3)
+        self.assertEqual(3, len(tenants))
 
         tenants_limited = client.tenants.list(limit=2)
-        self.assertEqual(len(tenants_limited), 2)
+        self.assertEqual(2, len(tenants_limited))
         self.assertEqual(tenants[0].name, tenants_limited[0].name)
         self.assertEqual(tenants[1].name, tenants_limited[1].name)
 
     def test_tenant_list_limit_bad_value(self):
+        from keystoneclient import exceptions as client_exceptions
+
         client = self.get_client()
         self.assertRaises(client_exceptions.BadRequest,
                           client.tenants.list, limit='a')
@@ -976,6 +1021,8 @@ class ClientDrivenTestCase(tests.TestCase):
         self.get_client(self.user_two)
 
     def test_user_cannot_update_other_users_passwd(self):
+        from keystoneclient import exceptions as client_exceptions
+
         client = self.get_client(self.user_two)
 
         token_id = client.auth_token
@@ -1003,6 +1050,8 @@ class ClientDrivenTestCase(tests.TestCase):
                           self.get_client, self.user_two)
 
     def test_tokens_after_user_update_passwd(self):
+        from keystoneclient import exceptions as client_exceptions
+
         client = self.get_client(self.user_two)
 
         token_id = client.auth_token
@@ -1031,3 +1080,28 @@ class ClientDrivenTestCase(tests.TestCase):
         self.assertRaises(client_exceptions.Unauthorized, client.tenants.list)
         client.auth_token = new_token_id
         client.tenants.list()
+
+
+class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
+    checkout_info = (KEYSTONECLIENT_REPO, 'master')
+
+    def setUp(self):
+        if os.environ.get('KSCTEST_PATH'):
+            self.skip('KSCTEST_PATH env set, running tests with local '
+                      'client instead.')
+        super(KcMasterTestCase, self).setUp()
+
+
+class KcOptTestCase(KcMasterTestCase):
+    # Set KSCTEST_PATH to the keystoneclient directory, then run this test.
+    #
+    # For example, to test your local keystoneclient,
+    #
+    # KSCTEST_PATH=/opt/stack/python-keystoneclient \
+    #  tox -e py27 test_keystoneclient.KcOptTestCase
+
+    def setUp(self):
+        self.checkout_info = os.environ.get('KSCTEST_PATH')
+        if not self.checkout_info:
+            self.skip('Set KSCTEST_PATH env to test with local client')
+        super(KcOptTestCase, self).setUp()
