@@ -15,18 +15,15 @@ import datetime
 import uuid
 
 import mock
+from oslo.utils import timeutils
 from testtools import matchers
 
 from keystone.common import dependency
-from keystone import config
 from keystone.contrib.revoke import model
 from keystone import exception
-from keystone.openstack.common import timeutils
 from keystone import tests
 from keystone.tests import test_backend_sql
-
-
-CONF = config.CONF
+from keystone.token import provider
 
 
 def _new_id():
@@ -80,12 +77,10 @@ def _matches(event, token_values):
 
     # The token has two attributes that can match the domain_id
     if event.domain_id is not None:
-        dom_id_matched = False
         for attribute_name in ['identity_domain_id', 'assignment_domain_id']:
             if event.domain_id == token_values[attribute_name]:
-                dom_id_matched = True
                 break
-        if not dom_id_matched:
+        else:
             return False
 
     if event.domain_scope_id is not None:
@@ -98,7 +93,7 @@ def _matches(event, token_values):
     # rest of the logic.
     attribute_names = ['project_id',
                        'expires_at', 'trust_id', 'consumer_id',
-                       'access_token_id']
+                       'access_token_id', 'audit_id', 'audit_chain_id']
     for attribute_name in attribute_names:
         if getattr(event, attribute_name) is not None:
             if (getattr(event, attribute_name) !=
@@ -107,12 +102,10 @@ def _matches(event, token_values):
 
     if event.role_id is not None:
         roles = token_values['roles']
-        role_found = False
         for role in roles:
             if event.role_id == role:
-                role_found = True
                 break
-        if not role_found:
+        else:
             return False
     if token_values['issued_at'] > event.issued_before:
         return False
@@ -172,7 +165,7 @@ class RevokeTests(object):
         # 'synchronize' should now be exercised and remove old expired events
         mock_utcnow.return_value = now_plus_2h
         self.revoke_api.revoke_by_expiration(_new_id(), now_plus_2h)
-        #should no longer throw an exception
+        # should no longer throw an exception
         self.revoke_api.check_token(token_values)
 
     def test_revoke_by_expiration_project_and_domain_fails(self):
@@ -275,6 +268,22 @@ class RevokeTreeTests(tests.TestCase):
         return self.tree.add_event(
             model.RevokeEvent(user_id=user_id))
 
+    def _revoke_by_audit_id(self, audit_id):
+        event = self.tree.add_event(
+            model.RevokeEvent(audit_id=audit_id))
+        self.events.append(event)
+        return event
+
+    def _revoke_by_audit_chain_id(self, audit_chain_id, project_id=None,
+                                  domain_id=None):
+        event = self.tree.add_event(
+            model.RevokeEvent(audit_chain_id=audit_chain_id,
+                              project_id=project_id,
+                              domain_id=domain_id)
+        )
+        self.events.append(event)
+        return event
+
     def _revoke_by_expiration(self, user_id, expires_at, project_id=None,
                               domain_id=None):
         event = self.tree.add_event(
@@ -363,6 +372,45 @@ class RevokeTreeTests(tests.TestCase):
         self.removeEvent(event)
         self._assertTokenNotRevoked(token_data_1)
 
+    def test_revoke_by_audit_id(self):
+        audit_id = provider.audit_info(parent_audit_id=None)[0]
+        token_data_1 = _sample_blank_token()
+        # Audit ID and Audit Chain ID are populated with the same value
+        # if the token is an original token
+        token_data_1['audit_id'] = audit_id
+        token_data_1['audit_chain_id'] = audit_id
+        event = self._revoke_by_audit_id(audit_id)
+        self._assertTokenRevoked(token_data_1)
+
+        audit_id_2 = provider.audit_info(parent_audit_id=audit_id)[0]
+        token_data_2 = _sample_blank_token()
+        token_data_2['audit_id'] = audit_id_2
+        token_data_2['audit_chain_id'] = audit_id
+        self._assertTokenNotRevoked(token_data_2)
+
+        self.removeEvent(event)
+        self._assertTokenNotRevoked(token_data_1)
+
+    def test_revoke_by_audit_chain_id(self):
+        audit_id = provider.audit_info(parent_audit_id=None)[0]
+        token_data_1 = _sample_blank_token()
+        # Audit ID and Audit Chain ID are populated with the same value
+        # if the token is an original token
+        token_data_1['audit_id'] = audit_id
+        token_data_1['audit_chain_id'] = audit_id
+        event = self._revoke_by_audit_chain_id(audit_id)
+        self._assertTokenRevoked(token_data_1)
+
+        audit_id_2 = provider.audit_info(parent_audit_id=audit_id)[0]
+        token_data_2 = _sample_blank_token()
+        token_data_2['audit_id'] = audit_id_2
+        token_data_2['audit_chain_id'] = audit_id
+        self._assertTokenRevoked(token_data_2)
+
+        self.removeEvent(event)
+        self._assertTokenNotRevoked(token_data_1)
+        self._assertTokenNotRevoked(token_data_2)
+
     def test_by_user_project(self):
         # When a user has a project-scoped token and the project-scoped token
         # is revoked then the token is revoked.
@@ -523,18 +571,24 @@ class RevokeTreeTests(tests.TestCase):
         self.assertEqual(turn + 1, len(self.tree.revoke_map
                                        ['trust_id=*']
                                        ['consumer_id=*']
-                                       ['access_token_id=*']))
+                                       ['access_token_id=*']
+                                       ['audit_id=*']
+                                       ['audit_chain_id=*']))
         # two different functions add  domain_ids, +1 for None
         self.assertEqual(2 * turn + 1, len(self.tree.revoke_map
                                            ['trust_id=*']
                                            ['consumer_id=*']
                                            ['access_token_id=*']
+                                           ['audit_id=*']
+                                           ['audit_chain_id=*']
                                            ['expires_at=*']))
         # two different functions add  project_ids, +1 for None
         self.assertEqual(2 * turn + 1, len(self.tree.revoke_map
                                            ['trust_id=*']
                                            ['consumer_id=*']
                                            ['access_token_id=*']
+                                           ['audit_id=*']
+                                           ['audit_chain_id=*']
                                            ['expires_at=*']
                                            ['domain_id=*']))
         # 10 users added
@@ -542,6 +596,8 @@ class RevokeTreeTests(tests.TestCase):
                                    ['trust_id=*']
                                    ['consumer_id=*']
                                    ['access_token_id=*']
+                                   ['audit_id=*']
+                                   ['audit_chain_id=*']
                                    ['expires_at=*']
                                    ['domain_id=*']
                                    ['project_id=*']))
@@ -553,10 +609,21 @@ class RevokeTreeTests(tests.TestCase):
         for i in range(0, 10):
             events.append(
                 self._revoke_by_user(_new_id()))
+
             args = (_new_id(),
                     expiry_base_time + datetime.timedelta(seconds=i))
             events.append(
                 self._revoke_by_expiration(*args))
+
+            self.assertEqual(i + 2, len(self.tree.revoke_map
+                                        ['trust_id=*']
+                                        ['consumer_id=*']
+                                        ['access_token_id=*']
+                                        ['audit_id=*']
+                                        ['audit_chain_id=*']),
+                             'adding %s to %s' % (args,
+                                                  self.tree.revoke_map))
+
             events.append(
                 self._revoke_by_project_role_assignment(_new_id(), _new_id()))
             events.append(
