@@ -34,6 +34,7 @@ To run these tests against a live database:
 from oslo.db.sqlalchemy import utils
 
 from keystone.contrib import endpoint_filter
+from keystone.contrib import endpoint_policy
 from keystone.contrib import example
 from keystone.contrib import federation
 from keystone.contrib import oauth1
@@ -61,11 +62,15 @@ class SqlUpgradeOAuth1Extension(test_sql_upgrade.SqlMigrateBase):
     def repo_package(self):
         return oauth1
 
-    def test_upgrade(self):
-        self.assertTableDoesNotExist('consumer')
-        self.assertTableDoesNotExist('request_token')
-        self.assertTableDoesNotExist('access_token')
-        self.upgrade(1, repository=self.repo_path)
+    def upgrade(self, version):
+        super(SqlUpgradeOAuth1Extension, self).upgrade(
+            version, repository=self.repo_path)
+
+    def downgrade(self, version):
+        super(SqlUpgradeOAuth1Extension, self).downgrade(
+            version, repository=self.repo_path)
+
+    def _assert_v1_3_tables(self):
         self.assertTableColumns('consumer',
                                 ['id',
                                  'description',
@@ -89,8 +94,7 @@ class SqlUpgradeOAuth1Extension(test_sql_upgrade.SqlMigrateBase):
                                  'consumer_id',
                                  'expires_at'])
 
-    def test_downgrade(self):
-        self.upgrade(1, repository=self.repo_path)
+    def _assert_v4_later_tables(self):
         self.assertTableColumns('consumer',
                                 ['id',
                                  'description',
@@ -102,7 +106,7 @@ class SqlUpgradeOAuth1Extension(test_sql_upgrade.SqlMigrateBase):
                                  'verifier',
                                  'authorizing_user_id',
                                  'requested_project_id',
-                                 'requested_roles',
+                                 'role_ids',
                                  'consumer_id',
                                  'expires_at'])
         self.assertTableColumns('access_token',
@@ -110,10 +114,34 @@ class SqlUpgradeOAuth1Extension(test_sql_upgrade.SqlMigrateBase):
                                  'access_secret',
                                  'authorizing_user_id',
                                  'project_id',
-                                 'requested_roles',
+                                 'role_ids',
                                  'consumer_id',
                                  'expires_at'])
-        self.downgrade(0, repository=self.repo_path)
+
+    def test_upgrade(self):
+        self.assertTableDoesNotExist('consumer')
+        self.assertTableDoesNotExist('request_token')
+        self.assertTableDoesNotExist('access_token')
+        self.upgrade(1)
+        self._assert_v1_3_tables()
+
+        # NOTE(blk-u): Migrations 2-3 don't modify the tables in a way that we
+        # can easily test for.
+
+        self.upgrade(4)
+        self._assert_v4_later_tables()
+
+        self.upgrade(5)
+        self._assert_v4_later_tables()
+
+    def test_downgrade(self):
+        self.upgrade(5)
+        self._assert_v4_later_tables()
+        self.downgrade(3)
+        self._assert_v1_3_tables()
+        self.downgrade(1)
+        self._assert_v1_3_tables()
+        self.downgrade(0)
         self.assertTableDoesNotExist('consumer')
         self.assertTableDoesNotExist('request_token')
         self.assertTableDoesNotExist('access_token')
@@ -137,6 +165,26 @@ class EndpointFilterExtension(test_sql_upgrade.SqlMigrateBase):
         self.assertTableDoesNotExist('project_endpoint')
 
 
+class EndpointPolicyExtension(test_sql_upgrade.SqlMigrateBase):
+    def repo_package(self):
+        return endpoint_policy
+
+    def test_upgrade(self):
+        self.assertTableDoesNotExist('policy_association')
+        self.upgrade(1, repository=self.repo_path)
+        self.assertTableColumns('policy_association',
+                                ['id', 'policy_id', 'endpoint_id',
+                                 'service_id', 'region_id'])
+
+    def test_downgrade(self):
+        self.upgrade(1, repository=self.repo_path)
+        self.assertTableColumns('policy_association',
+                                ['id', 'policy_id', 'endpoint_id',
+                                 'service_id', 'region_id'])
+        self.downgrade(0, repository=self.repo_path)
+        self.assertTableDoesNotExist('policy_association')
+
+
 class FederationExtension(test_sql_upgrade.SqlMigrateBase):
     """Test class for ensuring the Federation SQL."""
 
