@@ -37,8 +37,10 @@ from migrate.versioning import api as versioning_api
 from oslo.db import exception as db_exception
 from oslo.db.sqlalchemy import migration
 from oslo.db.sqlalchemy import session as db_session
-import sqlalchemy
+import six
+from sqlalchemy.engine import reflection
 import sqlalchemy.exc
+from sqlalchemy import schema
 
 from keystone.assignment.backends import sql as assignment_sql
 from keystone.common import sql
@@ -46,6 +48,7 @@ from keystone.common.sql import migrate_repo
 from keystone.common.sql import migration_helpers
 from keystone import config
 from keystone.contrib import federation
+from keystone.contrib import revoke
 from keystone import exception
 from keystone import tests
 from keystone.tests import default_fixtures
@@ -120,10 +123,14 @@ INITIAL_EXTENSION_TABLE_STRUCTURE = {
     'revocation_event': [
         'id', 'domain_id', 'project_id', 'user_id', 'role_id',
         'trust_id', 'consumer_id', 'access_token_id',
-        'issued_before', 'expires_at', 'revoked_at',
+        'issued_before', 'expires_at', 'revoked_at', 'audit_id',
+        'audit_chain_id',
     ],
 }
 
+EXTENSIONS = {'federation': federation,
+              'revoke': revoke}
+
 
 class SqlMigrateBase(tests.SQLDriverOverrides, tests.TestCase):
     def initialize_sql(self):
@@ -174,9 +181,30 @@ class SqlMigrateBase(tests.SQLDriverOverrides, tests.TestCase):
         meta = sqlalchemy.MetaData()
         meta.bind = self.engine
         meta.reflect(self.engine)
-        for table in list(meta.tables.keys()):
-            table = sqlalchemy.Table(table, meta, autoload=True)
-            table.drop(self.engine, checkfirst=True)
+
+        with self.engine.begin() as conn:
+            inspector = reflection.Inspector.from_engine(self.engine)
+            metadata = schema.MetaData()
+            tbs = []
+            all_fks = []
+
+            for table_name in inspector.get_table_names():
+                fks = []
+                for fk in inspector.get_foreign_keys(table_name):
+                    if not fk['name']:
+                        continue
+                    fks.append(
+                        schema.ForeignKeyConstraint((), (), name=fk['name']))
+                table = schema.Table(table_name, metadata, *fks)
+                tbs.append(table)
+                all_fks.extend(fks)
+
+            for fkc in all_fks:
+                conn.execute(schema.DropConstraint(fkc))
+
+            for table in tbs:
+                conn.execute(schema.DropTable(table))
+
         sql.cleanup()
         super(SqlMigrateBase, self).tearDown()
 
@@ -232,7 +260,10 @@ class SqlMigrateBase(tests.SQLDriverOverrides, tests.TestCase):
         self.initialize_sql()
         table = self.select_table(table_name)
         actual_cols = [col.name for col in table.columns]
-        self.assertEqual(expected_cols, actual_cols, '%s table' % table_name)
+        # Check if the columns are equal, but allow for a different order,
+        # which might occur after an upgrade followed by a downgrade
+        self.assertEqual(expected_cols.sort(), actual_cols.sort(),
+                         '%s table' % table_name)
 
     @property
     def initial_db_version(self):
@@ -753,6 +784,7 @@ class SqlUpgradeTests(SqlMigrateBase):
         # we should have 3 trusts in base
         self.assertEqual(3, session.query(trust_table).count())
 
+        session.close()
         self.downgrade(40)
         session = self.Session()
         trust_table = sqlalchemy.Table(
@@ -790,8 +822,6 @@ class SqlUpgradeTests(SqlMigrateBase):
     def test_upgrade_service_enabled_data(self):
         """Migration 44 has to migrate data from `extra` to `enabled`."""
 
-        session = self.Session()
-
         def add_service(**extra_data):
             service_id = uuid.uuid4().hex
 
@@ -806,6 +836,7 @@ class SqlUpgradeTests(SqlMigrateBase):
             return service_id
 
         self.upgrade(43)
+        session = self.Session()
 
         # Different services with expected enabled and extra values, and a
         # description.
@@ -834,7 +865,9 @@ class SqlUpgradeTests(SqlMigrateBase):
              (False, random_attr), random_attr_enabled_false_str),
         ]
 
+        session.close()
         self.upgrade(44)
+        session = self.Session()
 
         # Verify that the services have the expected values.
 
@@ -854,7 +887,7 @@ class SqlUpgradeTests(SqlMigrateBase):
 
             enabled, extra = fetch_service(service_id)
 
-            self.assertIs(exp_enabled, enabled, msg)
+            self.assertEqual(exp_enabled, enabled, msg)
             self.assertEqual(exp_extra, extra, msg)
 
     def test_downgrade_service_enabled_data(self):
@@ -865,8 +898,6 @@ class SqlUpgradeTests(SqlMigrateBase):
 
         """
 
-        session = self.Session()
-
         def add_service(enabled=True, **extra_data):
             service_id = uuid.uuid4().hex
 
@@ -882,6 +913,7 @@ class SqlUpgradeTests(SqlMigrateBase):
             return service_id
 
         self.upgrade(44)
+        session = self.Session()
 
         # Insert some services using the new format.
 
@@ -907,7 +939,9 @@ class SqlUpgradeTests(SqlMigrateBase):
              "enabled=False, something='whatever'"),
         ]
 
+        session.close()
         self.downgrade(43)
+        session = self.Session()
 
         # Verify that the services have the expected values.
 
@@ -954,8 +988,6 @@ class SqlUpgradeTests(SqlMigrateBase):
     def test_upgrade_endpoint_enabled_data(self):
         """Migration 42 has to migrate data from `extra` to `enabled`."""
 
-        session = self.Session()
-
         def add_service():
             service_id = uuid.uuid4().hex
 
@@ -983,6 +1015,7 @@ class SqlUpgradeTests(SqlMigrateBase):
             return endpoint_id
 
         self.upgrade(41)
+        session = self.Session()
 
         # Insert some endpoints using the old format where `enabled` is in
         # `extra` JSON.
@@ -1019,7 +1052,9 @@ class SqlUpgradeTests(SqlMigrateBase):
              (False, random_attr), random_attr_enabled_false_str),
         ]
 
+        session.close()
         self.upgrade(42)
+        session = self.Session()
 
         # Verify that the endpoints have the expected values.
 
@@ -1039,7 +1074,9 @@ class SqlUpgradeTests(SqlMigrateBase):
 
             enabled, extra = fetch_endpoint(endpoint_id)
 
-            self.assertIs(exp_enabled, enabled, msg)
+            # NOTE(henry-nash): Different databases may return enabled as a
+            # real boolean of 0/1 - so we use assertEqual not assertIs here.
+            self.assertEqual(exp_enabled, enabled, msg)
             self.assertEqual(exp_extra, extra, msg)
 
     def test_downgrade_endpoint_enabled_data(self):
@@ -1050,8 +1087,6 @@ class SqlUpgradeTests(SqlMigrateBase):
 
         """
 
-        session = self.Session()
-
         def add_service():
             service_id = uuid.uuid4().hex
 
@@ -1080,6 +1115,7 @@ class SqlUpgradeTests(SqlMigrateBase):
             return endpoint_id
 
         self.upgrade(42)
+        session = self.Session()
 
         # Insert some endpoints using the new format.
 
@@ -1104,7 +1140,9 @@ class SqlUpgradeTests(SqlMigrateBase):
              "enabled=False, something='whatever'"),
         ]
 
+        session.close()
         self.downgrade(41)
+        session = self.Session()
 
         # Verify that the endpoints have the expected values.
 
@@ -1132,7 +1170,6 @@ class SqlUpgradeTests(SqlMigrateBase):
         Create two regions with the same description.
 
         """
-        session = self.Session()
 
         def add_region():
             region_uuid = uuid.uuid4().hex
@@ -1146,6 +1183,7 @@ class SqlUpgradeTests(SqlMigrateBase):
             return region_uuid
 
         self.upgrade(43)
+        session = self.Session()
         # Write one region to the database
         add_region()
         # Write another region to the database with the same description
@@ -1160,7 +1198,6 @@ class SqlUpgradeTests(SqlMigrateBase):
         Create two regions with the same description.
 
         """
-        session = self.Session()
 
         def add_region(table):
             region_uuid = uuid.uuid4().hex
@@ -1180,6 +1217,7 @@ class SqlUpgradeTests(SqlMigrateBase):
 
         # Migrate to version 42
         self.upgrade(42)
+        session = self.Session()
         region_table = sqlalchemy.Table('region',
                                         get_metadata(),
                                         autoload=True)
@@ -1200,16 +1238,23 @@ class SqlUpgradeTests(SqlMigrateBase):
             # into more specific exception objects, we should catch both of
             # sqlalchemy and oslo.db exceptions. If an old oslo.db version
             # is installed, IntegrityError is raised. If >=0.4.0 version of
-            # oslo.db is installed, DBDuplicateEntry is raised.
+            # oslo.db is installed, DBError is raised.
             # When the global requirements is updated with
             # the version fixes exceptions wrapping, IntegrityError must be
             # removed from the tuple.
-            (sqlalchemy.exc.IntegrityError, db_exception.DBDuplicateEntry),
+
+            # NOTE(henry-nash): The above re-creation of the (now erased from
+            # history) unique constraint doesn't appear to work well with the
+            # Postgresql SQA driver, leading to it throwing a ValueError, so
+            # we also catch that here.
+            (sqlalchemy.exc.IntegrityError, db_exception.DBError, ValueError),
             add_region,
             table=region_unique_table)
 
         # migrate to 43, unique constraint should be dropped
+        session.close()
         self.upgrade(43)
+        session = self.Session()
 
         # reload the region table from the schema
         region_nonunique = sqlalchemy.Table('region',
@@ -1229,6 +1274,194 @@ class SqlUpgradeTests(SqlMigrateBase):
         self.downgrade(50)
         self.assertTableDoesNotExist('id_mapping')
 
+    def test_region_url_upgrade(self):
+        self.upgrade(52)
+        self.assertTableColumns('region',
+                                ['id', 'description', 'parent_region_id',
+                                 'extra', 'url'])
+
+    def test_region_url_downgrade(self):
+        self.upgrade(52)
+        self.downgrade(51)
+        self.assertTableColumns('region',
+                                ['id', 'description', 'parent_region_id',
+                                 'extra'])
+
+    def test_region_url_cleanup(self):
+        # make sure that the url field is dropped in the downgrade
+        self.upgrade(52)
+        session = self.Session()
+        beta = {
+            'id': uuid.uuid4().hex,
+            'description': uuid.uuid4().hex,
+            'parent_region_id': uuid.uuid4().hex,
+            'url': uuid.uuid4().hex
+        }
+        acme = {
+            'id': uuid.uuid4().hex,
+            'description': uuid.uuid4().hex,
+            'parent_region_id': uuid.uuid4().hex,
+            'url': None
+        }
+        self.insert_dict(session, 'region', beta)
+        self.insert_dict(session, 'region', acme)
+        region_table = sqlalchemy.Table('region', self.metadata, autoload=True)
+        self.assertEqual(2, session.query(region_table).count())
+        session.close()
+        self.downgrade(51)
+        session = self.Session()
+        self.metadata.clear()
+        region_table = sqlalchemy.Table('region', self.metadata, autoload=True)
+        self.assertEqual(2, session.query(region_table).count())
+        region = session.query(region_table)[0]
+        self.assertRaises(AttributeError, getattr, region, 'url')
+
+    def test_endpoint_region_upgrade_columns(self):
+        self.upgrade(53)
+        self.assertTableColumns('endpoint',
+                                ['id', 'legacy_endpoint_id', 'interface',
+                                 'service_id', 'url', 'extra', 'enabled',
+                                 'region_id'])
+        region_table = sqlalchemy.Table('region', self.metadata, autoload=True)
+        self.assertEqual(region_table.c.id.type.length, 255)
+        self.assertEqual(region_table.c.parent_region_id.type.length, 255)
+        endpoint_table = sqlalchemy.Table('endpoint',
+                                          self.metadata,
+                                          autoload=True)
+        self.assertEqual(endpoint_table.c.region_id.type.length, 255)
+
+    def test_endpoint_region_downgrade_columns(self):
+        self.upgrade(53)
+        self.downgrade(52)
+        self.assertTableColumns('endpoint',
+                                ['id', 'legacy_endpoint_id', 'interface',
+                                 'service_id', 'url', 'extra', 'enabled',
+                                 'region'])
+        region_table = sqlalchemy.Table('region', self.metadata, autoload=True)
+        self.assertEqual(region_table.c.id.type.length, 64)
+        self.assertEqual(region_table.c.parent_region_id.type.length, 64)
+        endpoint_table = sqlalchemy.Table('endpoint',
+                                          self.metadata,
+                                          autoload=True)
+        self.assertEqual(endpoint_table.c.region.type.length, 255)
+
+    def test_endpoint_region_migration(self):
+        self.upgrade(52)
+        session = self.Session()
+        _small_region_name = '0' * 30
+        _long_region_name = '0' * 255
+        _clashing_region_name = '0' * 70
+
+        def add_service():
+            service_id = uuid.uuid4().hex
+
+            service = {
+                'id': service_id,
+                'type': uuid.uuid4().hex
+            }
+
+            self.insert_dict(session, 'service', service)
+
+            return service_id
+
+        def add_endpoint(service_id, region):
+            endpoint_id = uuid.uuid4().hex
+
+            endpoint = {
+                'id': endpoint_id,
+                'interface': uuid.uuid4().hex[:8],
+                'service_id': service_id,
+                'url': uuid.uuid4().hex,
+                'region': region
+            }
+            self.insert_dict(session, 'endpoint', endpoint)
+
+            return endpoint_id
+
+        _service_id_ = add_service()
+        add_endpoint(_service_id_, region=_long_region_name)
+        add_endpoint(_service_id_, region=_long_region_name)
+        add_endpoint(_service_id_, region=_clashing_region_name)
+        add_endpoint(_service_id_, region=_small_region_name)
+        add_endpoint(_service_id_, region=None)
+
+        # upgrade to 53
+        session.close()
+        self.upgrade(53)
+        session = self.Session()
+        self.metadata.clear()
+
+        region_table = sqlalchemy.Table('region', self.metadata, autoload=True)
+        self.assertEqual(1, session.query(region_table).
+                         filter_by(id=_long_region_name).count())
+        self.assertEqual(1, session.query(region_table).
+                         filter_by(id=_clashing_region_name).count())
+        self.assertEqual(1, session.query(region_table).
+                         filter_by(id=_small_region_name).count())
+
+        endpoint_table = sqlalchemy.Table('endpoint',
+                                          self.metadata,
+                                          autoload=True)
+        self.assertEqual(5, session.query(endpoint_table).count())
+        self.assertEqual(2, session.query(endpoint_table).
+                         filter_by(region_id=_long_region_name).count())
+        self.assertEqual(1, session.query(endpoint_table).
+                         filter_by(region_id=_clashing_region_name).count())
+        self.assertEqual(1, session.query(endpoint_table).
+                         filter_by(region_id=_small_region_name).count())
+
+        # downgrade to 52
+        session.close()
+        self.downgrade(52)
+        session = self.Session()
+        self.metadata.clear()
+
+        region_table = sqlalchemy.Table('region', self.metadata, autoload=True)
+        self.assertEqual(1, session.query(region_table).count())
+        self.assertEqual(1, session.query(region_table).
+                         filter_by(id=_small_region_name).count())
+
+        endpoint_table = sqlalchemy.Table('endpoint',
+                                          self.metadata,
+                                          autoload=True)
+        self.assertEqual(5, session.query(endpoint_table).count())
+        self.assertEqual(2, session.query(endpoint_table).
+                         filter_by(region=_long_region_name).count())
+        self.assertEqual(1, session.query(endpoint_table).
+                         filter_by(region=_clashing_region_name).count())
+        self.assertEqual(1, session.query(endpoint_table).
+                         filter_by(region=_small_region_name).count())
+
+    def test_add_actor_id_index(self):
+        self.upgrade(53)
+        self.upgrade(54)
+        table = sqlalchemy.Table('assignment', self.metadata, autoload=True)
+        index_data = [(idx.name, idx.columns.keys()) for idx in table.indexes]
+        self.assertIn(('ix_actor_id', ['actor_id']), index_data)
+
+    def test_token_user_id_and_trust_id_index_upgrade(self):
+        self.upgrade(54)
+        self.upgrade(55)
+        table = sqlalchemy.Table('token', self.metadata, autoload=True)
+        index_data = [(idx.name, idx.columns.keys()) for idx in table.indexes]
+        self.assertIn(('ix_token_user_id', ['user_id']), index_data)
+        self.assertIn(('ix_token_trust_id', ['trust_id']), index_data)
+
+    def test_token_user_id_and_trust_id_index_downgrade(self):
+        self.upgrade(55)
+        self.downgrade(54)
+        table = sqlalchemy.Table('token', self.metadata, autoload=True)
+        index_data = [(idx.name, idx.columns.keys()) for idx in table.indexes]
+        self.assertNotIn(('ix_token_user_id', ['user_id']), index_data)
+        self.assertNotIn(('ix_token_trust_id', ['trust_id']), index_data)
+
+    def test_remove_actor_id_index(self):
+        self.upgrade(54)
+        self.downgrade(53)
+        table = sqlalchemy.Table('assignment', self.metadata, autoload=True)
+        index_data = [(idx.name, idx.columns.keys()) for idx in table.indexes]
+        self.assertNotIn(('ix_actor_id', ['actor_id']), index_data)
+
     def populate_user_table(self, with_pass_enab=False,
                             with_pass_enab_domain=False):
         # Populate the appropriate fields in the user
@@ -1374,18 +1607,38 @@ class VersionTests(SqlMigrateBase):
 
     def test_extension_initial(self):
         """When get the initial version of an extension, it's 0."""
-        abs_path = migration_helpers.find_migrate_repo(federation)
-        migration.db_version_control(sql.get_engine(), abs_path)
-        version = migration_helpers.get_db_version(extension='federation')
-        self.assertEqual(0, version)
+        for name, extension in six.iteritems(EXTENSIONS):
+            abs_path = migration_helpers.find_migrate_repo(extension)
+            migration.db_version_control(sql.get_engine(), abs_path)
+            version = migration_helpers.get_db_version(extension=name)
+            self.assertEqual(0, version,
+                             'Migrate version for %s is not 0' % name)
 
     def test_extension_migrated(self):
         """When get the version after migrating an extension, it's not 0."""
-        abs_path = migration_helpers.find_migrate_repo(federation)
-        migration.db_version_control(sql.get_engine(), abs_path)
-        migration.db_sync(sql.get_engine(), abs_path)
-        version = migration_helpers.get_db_version(extension='federation')
-        self.assertTrue(version > 0, "Version didn't change after migrated?")
+        for name, extension in six.iteritems(EXTENSIONS):
+            abs_path = migration_helpers.find_migrate_repo(extension)
+            migration.db_version_control(sql.get_engine(), abs_path)
+            migration.db_sync(sql.get_engine(), abs_path)
+            version = migration_helpers.get_db_version(extension=name)
+            self.assertTrue(
+                version > 0,
+                "Version for %s didn't change after migrated?" % name)
+
+    def test_extension_downgraded(self):
+        """When get the version after downgrading an extension, it is 0."""
+        for name, extension in six.iteritems(EXTENSIONS):
+            abs_path = migration_helpers.find_migrate_repo(extension)
+            migration.db_version_control(sql.get_engine(), abs_path)
+            migration.db_sync(sql.get_engine(), abs_path)
+            version = migration_helpers.get_db_version(extension=name)
+            self.assertTrue(
+                version > 0,
+                "Version for %s didn't change after migrated?" % name)
+            migration.db_sync(sql.get_engine(), abs_path, version=0)
+            version = migration_helpers.get_db_version(extension=name)
+            self.assertEqual(0, version,
+                             'Migrate version for %s is not 0' % name)
 
     def test_unexpected_extension(self):
         """The version for an extension that doesn't exist raises ImportError.
