@@ -10,21 +10,25 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from oslo.config import fixture as config_fixture
+import testtools
+
 from keystone.catalog import core
 from keystone import config
 from keystone import exception
-from keystone import tests
 
 
 CONF = config.CONF
 
 
-class FormatUrlTests(tests.TestCase):
+class FormatUrlTests(testtools.TestCase):
 
     def setUp(self):
         super(FormatUrlTests, self).setUp()
-        whitelist = ['host', 'port', 'part1', 'part2']
-        CONF.catalog.endpoint_substitution_whitelist = whitelist
+        fixture = self.useFixture(config_fixture.Config(CONF))
+        fixture.config(
+            group='catalog',
+            endpoint_substitution_whitelist=['host', 'port', 'part1', 'part2'])
 
     def test_successful_formatting(self):
         url_template = 'http://%(host)s:%(port)d/%(part1)s/%(part2)s'
