@@ -18,14 +18,16 @@ import copy
 from testtools import matchers
 
 from keystone.common import json_home
-from keystone.tests import unit
+from keystone import tests
 
 
-class JsonHomeTest(unit.BaseTestCase):
+class JsonHomeTest(tests.BaseTestCase):
     def test_build_v3_resource_relation(self):
         resource_name = self.getUniqueString()
         relation = json_home.build_v3_resource_relation(resource_name)
-        exp_relation = f'https://docs.openstack.org/api/openstack-identity/3/rel/{resource_name}'
+        exp_relation = (
+            'http://docs.openstack.org/api/openstack-identity/3/rel/%s' %
+            resource_name)
         self.assertThat(relation, matchers.Equals(exp_relation))
 
     def test_build_v3_extension_resource_relation(self):
@@ -33,18 +35,18 @@ class JsonHomeTest(unit.BaseTestCase):
         extension_version = self.getUniqueString()
         resource_name = self.getUniqueString()
         relation = json_home.build_v3_extension_resource_relation(
-            extension_name, extension_version, resource_name
-        )
+            extension_name, extension_version, resource_name)
         exp_relation = (
-            f'https://docs.openstack.org/api/openstack-identity/3/ext/{extension_name}/{extension_version}/rel'
-            f'/{resource_name}'
-        )
+            'http://docs.openstack.org/api/openstack-identity/3/ext/%s/%s/rel/'
+            '%s' % (extension_name, extension_version, resource_name))
         self.assertThat(relation, matchers.Equals(exp_relation))
 
     def test_build_v3_parameter_relation(self):
         parameter_name = self.getUniqueString()
         relation = json_home.build_v3_parameter_relation(parameter_name)
-        exp_relation = f'https://docs.openstack.org/api/openstack-identity/3/param/{parameter_name}'
+        exp_relation = (
+            'http://docs.openstack.org/api/openstack-identity/3/param/%s' %
+            parameter_name)
         self.assertThat(relation, matchers.Equals(exp_relation))
 
     def test_build_v3_extension_parameter_relation(self):
@@ -52,12 +54,10 @@ class JsonHomeTest(unit.BaseTestCase):
         extension_version = self.getUniqueString()
         parameter_name = self.getUniqueString()
         relation = json_home.build_v3_extension_parameter_relation(
-            extension_name, extension_version, parameter_name
-        )
+            extension_name, extension_version, parameter_name)
         exp_relation = (
-            f'https://docs.openstack.org/api/openstack-identity/3/ext/{extension_name}/{extension_version}/'
-            f'param/{parameter_name}'
-        )
+            'http://docs.openstack.org/api/openstack-identity/3/ext/%s/%s/'
+            'param/%s' % (extension_name, extension_version, parameter_name))
         self.assertThat(relation, matchers.Equals(exp_relation))
 
     def test_translate_urls(self):
@@ -71,8 +71,7 @@ class JsonHomeTest(unit.BaseTestCase):
                 href_rel: {'href': href},
                 href_template_rel: {
                     'href-template': href_template,
-                    'href-vars': href_vars,
-                },
+                    'href-vars': href_vars}
             }
         }
 
@@ -85,8 +84,7 @@ class JsonHomeTest(unit.BaseTestCase):
                 href_rel: {'href': new_prefix + href},
                 href_template_rel: {
                     'href-template': new_prefix + href_template,
-                    'href-vars': href_vars,
-                },
+                    'href-vars': href_vars}
             }
         }
 
