@@ -13,10 +13,11 @@
 import copy
 import uuid
 
+from oslo.utils import timeutils
+
 from keystone.common import config
 from keystone import exception
 from keystone.models import token_model
-from keystone.openstack.common import timeutils
 from keystone.tests import core
 from keystone.tests import test_token_provider
 
@@ -101,6 +102,61 @@ class TestKeystoneTokenModel(core.TestCase):
         self.assertEqual(domain_name, token_data.domain_name)
         self.assertTrue(token_data.domain_scoped)
 
+        token_data['audit_ids'] = [uuid.uuid4().hex]
+        self.assertEqual(token_data.audit_id,
+                         token_data['audit_ids'][0])
+        self.assertEqual(token_data.audit_chain_id,
+                         token_data['audit_ids'][0])
+        token_data['audit_ids'].append(uuid.uuid4().hex)
+        self.assertEqual(token_data.audit_chain_id,
+                         token_data['audit_ids'][1])
+        del token_data['audit_ids']
+        self.assertIsNone(token_data.audit_id)
+        self.assertIsNone(token_data.audit_chain_id)
+
+    def test_token_model_v3_federated_user(self):
+        token_data = token_model.KeystoneToken(token_id=uuid.uuid4().hex,
+                                               token_data=self.v3_sample_token)
+        federation_data = {'identity_provider': {'id': uuid.uuid4().hex},
+                           'protocol': {'id': 'saml2'},
+                           'groups': [{'id': uuid.uuid4().hex}
+                                      for x in range(1, 5)]}
+
+        self.assertFalse(token_data.is_federated_user)
+        self.assertEqual([], token_data.federation_group_ids)
+        self.assertIsNone(token_data.federation_protocol_id)
+        self.assertIsNone(token_data.federation_idp_id)
+
+        token_data['user'][token_model.federation.FEDERATION] = federation_data
+
+        self.assertTrue(token_data.is_federated_user)
+        self.assertEqual([x['id'] for x in federation_data['groups']],
+                         token_data.federation_group_ids)
+        self.assertEqual(federation_data['protocol']['id'],
+                         token_data.federation_protocol_id)
+        self.assertEqual(federation_data['identity_provider']['id'],
+                         token_data.federation_idp_id)
+
+    def test_token_model_v2_federated_user(self):
+        token_data = token_model.KeystoneToken(token_id=uuid.uuid4().hex,
+                                               token_data=self.v2_sample_token)
+        federation_data = {'identity_provider': {'id': uuid.uuid4().hex},
+                           'protocol': {'id': 'saml2'},
+                           'groups': [{'id': uuid.uuid4().hex}
+                                      for x in range(1, 5)]}
+        self.assertFalse(token_data.is_federated_user)
+        self.assertEqual([], token_data.federation_group_ids)
+        self.assertIsNone(token_data.federation_protocol_id)
+        self.assertIsNone(token_data.federation_idp_id)
+
+        token_data['user'][token_model.federation.FEDERATION] = federation_data
+
+        # Federated users should not exist in V2, the data should remain empty
+        self.assertFalse(token_data.is_federated_user)
+        self.assertEqual([], token_data.federation_group_ids)
+        self.assertIsNone(token_data.federation_protocol_id)
+        self.assertIsNone(token_data.federation_idp_id)
+
     def test_token_model_v2(self):
         token_data = token_model.KeystoneToken(uuid.uuid4().hex,
                                                self.v2_sample_token)
@@ -124,7 +180,12 @@ class TestKeystoneTokenModel(core.TestCase):
                          token_data.project_domain_name)
         self.assertEqual(self.v2_sample_token['access']['trust']['id'],
                          token_data.trust_id)
-        self.assertIsNone(token_data.trustor_user_id)
+        self.assertEqual(
+            self.v2_sample_token['access']['trust']['trustor_user_id'],
+            token_data.trustor_user_id)
+        self.assertEqual(
+            self.v2_sample_token['access']['trust']['impersonation'],
+            token_data.trust_impersonation)
         self.assertEqual(
             self.v2_sample_token['access']['trust']['trustee_user_id'],
             token_data.trustee_user_id)
@@ -167,6 +228,18 @@ class TestKeystoneTokenModel(core.TestCase):
                           'domain_name')
         self.assertFalse(token_data.domain_scoped)
 
+        token_data['token']['audit_ids'] = [uuid.uuid4().hex]
+        self.assertEqual(token_data.audit_chain_id,
+                         token_data['token']['audit_ids'][0])
+        token_data['token']['audit_ids'].append(uuid.uuid4().hex)
+        self.assertEqual(token_data.audit_chain_id,
+                         token_data['token']['audit_ids'][1])
+        self.assertEqual(token_data.audit_id,
+                         token_data['token']['audit_ids'][0])
+        del token_data['token']['audit_ids']
+        self.assertIsNone(token_data.audit_id)
+        self.assertIsNone(token_data.audit_chain_id)
+
     def test_token_model_unknown(self):
         self.assertRaises(exception.UnsupportedTokenVersionException,
                           token_model.KeystoneToken,
