@@ -13,4 +13,4 @@
 from keystone.token.persistence.core import *  # noqa
 
 
-__all__ = ('PersistenceManager', 'TokenDriverBase')
+__all__ = ['Manager', 'Driver', 'backends']
