@@ -13,24 +13,17 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from oslo_log import versionutils
-
-import keystone.conf
+from keystone.common import config
 from keystone.token.persistence.backends import kvs
 
 
-CONF = keystone.conf.CONF
+CONF = config.CONF
 
 
 class Token(kvs.Token):
     kvs_backend = 'openstack.kvs.Memcached'
     memcached_backend = 'memcached'
 
-    @versionutils.deprecated(
-        what='Memcache Token Persistence Driver',
-        as_of=versionutils.deprecated.MITAKA,
-        in_favor_of='fernet token driver (no-persistence)',
-        remove_in=0)
     def __init__(self, *args, **kwargs):
         kwargs['memcached_backend'] = self.memcached_backend
         kwargs['no_expiry_keys'] = [self.revocation_key]
