@@ -14,10 +14,11 @@
 
 import time
 
+from oslo.utils import timeutils
+
 from keystone.common import sql
 from keystone import exception
 from keystone.openstack.common import log
-from keystone.openstack.common import timeutils
 from keystone import trust
 
 
@@ -31,11 +32,11 @@ class TrustModel(sql.ModelBase, sql.DictBase):
     __tablename__ = 'trust'
     attributes = ['id', 'trustor_user_id', 'trustee_user_id',
                   'project_id', 'impersonation', 'expires_at',
-                  'remaining_uses']
+                  'remaining_uses', 'deleted_at']
     id = sql.Column(sql.String(64), primary_key=True)
-    #user id Of owner
+    # user id of owner
     trustor_user_id = sql.Column(sql.String(64), nullable=False,)
-    #user_id of user allowed to consume this preauth
+    # user_id of user allowed to consume this preauth
     trustee_user_id = sql.Column(sql.String(64), nullable=False)
     project_id = sql.Column(sql.String(64))
     impersonation = sql.Column(sql.Boolean, nullable=False)
@@ -55,8 +56,7 @@ class TrustRole(sql.ModelBase):
 class Trust(trust.Driver):
     @sql.handle_conflicts(conflict_type='trust')
     def create_trust(self, trust_id, trust, roles):
-        session = sql.get_session()
-        with session.begin():
+        with sql.transaction() as session:
             ref = TrustModel.from_dict(trust)
             ref['id'] = trust_id
             if ref.get('expires_at') and ref['expires_at'].tzinfo is not None:
@@ -81,6 +81,7 @@ class Trust(trust.Driver):
 
     @sql.handle_conflicts(conflict_type='trust')
     def consume_use(self, trust_id):
+
         for attempt in range(MAXIMUM_CONSUME_ATTEMPTS):
             with sql.transaction() as session:
                 try:
@@ -127,19 +128,20 @@ class Trust(trust.Driver):
             # incorrectly indicating a trust was consumed.
             raise exception.TrustConsumeMaximumAttempt(trust_id=trust_id)
 
-    def get_trust(self, trust_id):
+    def get_trust(self, trust_id, deleted=False):
         session = sql.get_session()
-        ref = (session.query(TrustModel).
-               filter_by(deleted_at=None).
-               filter_by(id=trust_id).first())
+        query = session.query(TrustModel).filter_by(id=trust_id)
+        if not deleted:
+            query = query.filter_by(deleted_at=None)
+        ref = query.first()
         if ref is None:
             return None
-        if ref.expires_at is not None:
+        if ref.expires_at is not None and not deleted:
             now = timeutils.utcnow()
             if now > ref.expires_at:
                 return None
         # Do not return trusts that can't be used anymore
-        if ref.remaining_uses is not None:
+        if ref.remaining_uses is not None and not deleted:
             if ref.remaining_uses <= 0:
                 return None
         trust_dict = ref.to_dict()
@@ -171,8 +173,7 @@ class Trust(trust.Driver):
 
     @sql.handle_conflicts(conflict_type='trust')
     def delete_trust(self, trust_id):
-        session = sql.get_session()
-        with session.begin():
+        with sql.transaction() as session:
             trust_ref = session.query(TrustModel).get(trust_id)
             if not trust_ref:
                 raise exception.TrustNotFound(trust_id=trust_id)
